@@ -9,7 +9,7 @@
 
 /* Synched up with: FSF 19.28. */
 
-#include <config.h>
+#include <../src/config.h>
 
 #include <stdio.h>
 #include <ctype.h>
@@ -44,10 +44,8 @@ void setup_yow (FILE *fp);
     &res;})
 #endif
 
-void
-main (argc, argv)
-     int argc;
-     char *argv[];
+int
+main (int argc, char *argv[])
 {
   FILE *fp;
   char file[BUFSIZ];
@@ -82,7 +80,7 @@ main (argc, argv)
   setup_yow(fp);
   yow(fp);
   fclose(fp);
-  exit(0);
+  return 0;
 }
 
 static long len = -1;
