@@ -1,5 +1,9 @@
-;; Record version number of Emacs.
-;; Copyright (C) 1985, 1991-1994 Free Software Foundation, Inc.
+;; version.el --- Record version number of Emacs.
+
+;; Copyright (C) 1985, 1991-1994, 1997 Free Software Foundation, Inc.
+
+;; Maintainer: XEmacs Development Team
+;; Keywords: internal, dumped
 
 ;; This file is part of XEmacs.
 
@@ -20,30 +24,53 @@
 
 ;;; Synched up with: FSF 19.34.
 
+;;; Commentary:
+
+;; This file is dumped with XEmacs.
+
 ;;; Code:
 
-(defconst emacs-version "20.2"
-  "Version numbers of this version of Emacs.")
+(defconst xemacs-betaname
+  (and emacs-beta-version (format "(beta%d)" emacs-beta-version))
+  "Non-nil when this is a test (beta) version of XEmacs.
+Warning, this variable did not exist in XEmacs versions prior to 20.3")
 
-(setq emacs-version (purecopy (concat emacs-version " XEmacs Lucid")))
+(defconst emacs-version
+  (purecopy
+   (format "%d.%d %s%s%s"
+	   emacs-major-version
+	   emacs-minor-version
+	   (if xemacs-codename
+	       (concat "\"" xemacs-codename "\"")
+	     "")
+	   " XEmacs Lucid"
+	   (if xemacs-betaname
+	       (concat " " xemacs-betaname)
+	     "")))
+  "Version numbers of this version of XEmacs.")
 
-(defconst emacs-major-version
-  (progn (or (string-match "^[0-9]+" emacs-version)
-	     (error "emacs-version unparsable"))
-         (string-to-int (match-string 0 emacs-version)))
-  "Major version number of this version of Emacs, as an integer.
-Warning, this variable did not exist in Emacs versions earlier than:
-  FSF Emacs:   19.23
-  XEmacs:      19.10")
+(if (featurep 'infodock)
+    (require 'id-vers))
 
-(defconst emacs-minor-version
-  (progn (or (string-match "^[0-9]+\\.\\([0-9]+\\)" emacs-version)
-	     (error "emacs-version unparsable"))
-         (string-to-int (match-string 1 emacs-version)))
-  "Minor version number of this version of Emacs, as an integer.
-Warning, this variable did not exist in Emacs versions earlier than:
-  FSF Emacs:   19.23
-  XEmacs:      19.10")
+;; Moved to C code as of XEmacs 20.3
+;(defconst emacs-major-version
+;  (progn (or (string-match "^[0-9]+" emacs-version)
+;	     (error "emacs-version unparsable"))
+;         (string-to-int (match-string 0 emacs-version)))
+;  "Major version number of this version of Emacs, as an integer.
+;Warning, this variable did not exist in Emacs versions earlier than:
+;  FSF Emacs:   19.23
+;  XEmacs:      19.10")
+
+;; Moved to C code as of XEmacs 20.3
+;(defconst emacs-minor-version
+;  (progn (or (string-match "^[0-9]+\\.\\([0-9]+\\)" emacs-version)
+;	     (error "emacs-version unparsable"))
+;         (string-to-int (match-string 1 emacs-version)))
+;  "Minor version number of this version of Emacs, as an integer.
+;Warning, this variable did not exist in Emacs versions earlier than:
+;  FSF Emacs:   19.23
+;  XEmacs:      19.10")
 
 (defconst emacs-build-time (current-time-string)
   "Time at which Emacs was dumped out.")
@@ -56,24 +83,28 @@ When called interactively with a prefix argument, insert string at point.
 Don't use this function in programs to choose actions according
 to the system configuration; look at `system-configuration' instead."
   (interactive "p")
-  (let ((version-string
-         (format
-	  "XEmacs %s [Lucid] (%s%s) of %s %s on %s"
-	  (substring emacs-version 0 (string-match " XEmacs" emacs-version))
-	  system-configuration
-	  (cond ((or (and (fboundp 'featurep)
-			  (featurep 'mule))
-		     (memq 'mule features)) ", Mule")
-		(t ""))
-	  (substring emacs-build-time 0
-		     (string-match " *[0-9]*:" emacs-build-time))
-	  (substring emacs-build-time
-		     (string-match "[0-9]*$" emacs-build-time))
-	  emacs-build-system)))
-    (cond
-     ((null arg) version-string)
-     ((eq arg 1) (message "%s" version-string))
-     (t          (insert version-string)))))
+  (save-match-data
+    (let ((version-string
+	   (format
+	    "XEmacs %s %s(%s%s) of %s %s on %s"
+	    (substring emacs-version 0 (string-match " XEmacs" emacs-version))
+	    (if (not (featurep 'infodock))
+		"[Lucid] "
+	      "")
+	    system-configuration
+	    (cond ((or (and (fboundp 'featurep)
+			    (featurep 'mule))
+		       (memq 'mule features)) ", Mule")
+		  (t ""))
+	    (substring emacs-build-time 0
+		       (string-match " *[0-9]*:" emacs-build-time))
+	    (substring emacs-build-time
+		       (string-match "[0-9]*$" emacs-build-time))
+	    emacs-build-system)))
+      (cond
+       ((null arg) version-string)
+       ((eq arg 1) (message "%s" version-string))
+       (t          (insert version-string))))))
 
 ;; from emacs-vers.el
 (defun emacs-version>= (major &optional minor)
