@@ -27,17 +27,13 @@ Boston, MA 02111-1307, USA.  */
 #include <ctype.h>
 #include <stdio.h>
 #include <sys/types.h>
-#ifdef HAVE_LIMITS_H
 #include <limits.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#if 0 /* mrb */
-#include <X11/Xos.h>
-#endif
 #include <X11/IntrinsicP.h>
+#include <X11/ShellP.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
 #include <X11/bitmaps/gray>
@@ -100,6 +96,10 @@ xlwMenuResources[] =
      offset(menu.foreground), XtRString, "XtDefaultForeground"},
   {XtNbuttonForeground, XtCButtonForeground, XtRPixel, sizeof(Pixel),
      offset(menu.button_foreground), XtRString, "XtDefaultForeground"},
+  {XtNhighlightForeground, XtCHighlightForeground, XtRPixel, sizeof(Pixel),
+     offset(menu.highlight_foreground), XtRString, "XtDefaultForeground"},
+  {XtNtitleForeground, XtCTitleForeground, XtRPixel, sizeof(Pixel),
+     offset(menu.title_foreground), XtRString, "XtDefaultForeground"},
   {XtNmargin, XtCMargin, XtRDimension,  sizeof(Dimension),
      offset(menu.margin), XtRImmediate, (XtPointer)2},
   {XmNmarginWidth, XmCMarginWidth, XmRHorizontalDimension, sizeof(Dimension),
@@ -110,9 +110,15 @@ xlwMenuResources[] =
      offset(menu.column_spacing), XtRImmediate, (XtPointer)4},
   {XmNindicatorSize, XmCIndicatorSize, XtRDimension,  sizeof(Dimension),
      offset(menu.indicator_size), XtRImmediate, (XtPointer)0},
+#if 0
   {XmNshadowThickness, XmCShadowThickness, XmRHorizontalDimension,
      sizeof (Dimension), offset (menu.shadow_thickness),
      XtRImmediate, (XtPointer) 2},
+#else
+  {XmNshadowThickness, XmCShadowThickness, XtRDimension,
+     sizeof (Dimension), offset (menu.shadow_thickness),
+     XtRImmediate, (XtPointer) 2},
+#endif
   {XmNselectColor, XmCSelectColor, XtRPixel, sizeof (Pixel),
      offset (menu.select_color), XtRImmediate, (XtPointer)-1},
   {XmNtopShadowColor, XmCTopShadowColor, XtRPixel, sizeof (Pixel),
@@ -336,7 +342,7 @@ make_old_stack_space (XlwMenuWidget mw, int n)
 }
 
 static Boolean
-close_to_reference_time(Widget w, Time reference_time, XEvent *ev)
+close_to_reference_time (Widget w, Time reference_time, XEvent *ev)
 {
   return
     reference_time &&
@@ -374,10 +380,10 @@ string_width (XlwMenuWidget mw,
 static char massaged_resource_char[256];
 
 static void
-Initialize_massaged_resource_char (void)
+initialize_massaged_resource_char (void)
 {
   int j;
-  for (j = 0; j < sizeof (massaged_resource_char); j++)
+  for (j = 0; j < (int) sizeof (massaged_resource_char); j++)
     {
       if ((j >= 'a' && j <= 'z') ||
 	  (j >= 'A' && j <= 'Z') ||
@@ -411,7 +417,8 @@ string_width_u (XlwMenuWidget mw,
   int drop;
 # endif
 #endif
-  char newchars[64];
+  char* newchars;
+  int charslength;
   char *chars;
   int i, j;
 
@@ -423,8 +430,10 @@ string_width_u (XlwMenuWidget mw,
 #else
   chars = string;
 #endif
+  charslength = strlen (chars);
+  newchars = (char *) alloca (charslength + 1);
 
-  for (i = j = 0; chars[i] && (j < sizeof (newchars)); i++)
+  for (i = j = 0; chars[i] && (j < charslength); i++)
     if (chars[i]=='%'&&chars[i+1]=='_')
 	    i++;
     else
@@ -434,11 +443,12 @@ string_width_u (XlwMenuWidget mw,
 #ifdef NEED_MOTIF
   newstring = XmStringLtoRCreate (newchars, XmFONTLIST_DEFAULT_TAG);
   XmStringExtent (mw->menu.font_list, newstring, &width, &height);
-  XmStringFree(newstring);
+  XmStringFree (newstring);
+  XtFree (chars);
   return width;
 #else
 # ifdef USE_XFONTSET
-  XmbTextExtents(mw->menu.font_set, newchars, j, &ri, &rl);
+  XmbTextExtents (mw->menu.font_set, newchars, j, &ri, &rl);
   return rl.width;
 # else /* ! USE_XFONTSET */
   XTextExtents (mw->menu.font, newchars, j, &drop, &drop, &drop, &xcs);
@@ -473,7 +483,8 @@ massage_resource_name (CONST char *in, char *out)
       char ch = massaged_resource_char[(unsigned char) *in++];
       if (ch)
 	{
-	  *out++ = firstp ? tolower (ch) : toupper (ch);
+	  int int_ch = (int) (unsigned char) ch;
+	  *out++ = firstp ? tolower (int_ch) : toupper (int_ch);
 	  firstp = False;
 	  while ((ch = massaged_resource_char[(unsigned char) *in++]) != '\0')
 	    *out++ = ch;
@@ -484,8 +495,8 @@ massage_resource_name (CONST char *in, char *out)
   *out = 0;
 
 #ifdef PRINT_XLWMENU_RESOURCE_CONVERSIONS
-  printf("! Emacs*XlwMenu.%s.labelString:\t%s\n", save_out, save_in);
-  printf(  "Emacs*XlwMenu.%s.labelString:\n",     save_out);
+  printf ("! Emacs*XlwMenu.%s.labelString:\t%s\n", save_out, save_in);
+  printf (  "Emacs*XlwMenu.%s.labelString:\n",     save_out);
 #endif
 }
 
@@ -514,8 +525,8 @@ parameterize_string (CONST char *string, CONST char *value)
 {
   char *percent;
   char *result;
-  unsigned done = 0;
-  unsigned ntimes;
+  unsigned int done = 0;
+  unsigned int ntimes;
 
   if (!string)
     {
@@ -536,13 +547,13 @@ parameterize_string (CONST char *string, CONST char *value)
 
   while ((percent = strchr(string, '%')))
     {
-      unsigned left_pad;
-      unsigned right_pad;
+      unsigned int left_pad;
+      unsigned int right_pad;
       char *p;
 
       if (percent[1] == '%')
 	{	/* it's a real % */
-	  strncat(result, string, 1 + percent - string); /* incl % */
+	  strncat (result, string, 1 + percent - string); /* incl % */
 	  string = &percent[2];	/* after the second '%' */
 	  continue;		/* with the while() loop */
 	}
@@ -562,15 +573,15 @@ parameterize_string (CONST char *string, CONST char *value)
 	    }
 	  else if (*p == '1')
 	    {			/* param and terminator */
-	      strncat(result, string, percent - string);
+	      strncat (result, string, percent - string);
 	      if (value[0] != '\0')
 		{
-		  unsigned i;
+		  unsigned int i;
 		  for (i = 0; i < left_pad; i++)
-		    strcat(result, " ");
-		  strcat(result, value);
+		    strcat (result, " ");
+		  strcat (result, value);
 		  for (i = 0; i < right_pad; i++)
-		    strcat(result, " ");
+		    strcat (result, " ");
 		}
 	      string = &p[1];	/* after the '1' */
 	      done++;		/* no need to do old way */
@@ -579,25 +590,21 @@ parameterize_string (CONST char *string, CONST char *value)
 	  else
 	    {			/* bogus, copy the format as is */
 				/* out of for() loop */
-	      strncat(result, string, 1 + p - string);
-	      string= (*p ? &p[1] : p);
+	      strncat (result, string, 1 + p - string);
+	      string = (*p ? &p[1] : p);
 	      break;
 	    }
 	}
     }
 
-  /*
-   *    Copy the tail of the string
-   */
-  strcat(result, string);
+  /* Copy the tail of the string */
+  strcat (result, string);
 
-  /*
-   *    If we have not processed a % string, and we have a value, tail it.
-   */
+  /* If we have not processed a % string, and we have a value, tail it. */
   if (!done && value[0] != '\0')
     {
-      strcat(result, " ");
-      strcat(result, value);
+      strcat (result, " ");
+      strcat (result, value);
     }
 
   return result;
@@ -626,7 +633,7 @@ resource_widget_value (XlwMenuWidget mw, widget_value *val)
 	  if (val->value)
 	    {
 	      char named_name[1024];
-	      sprintf(named_name, "%sNamed", massaged_name);
+	      sprintf (named_name, "%sNamed", massaged_name);
 	      XtGetSubresources ((Widget) mw,
 				 (XtPointer) &resourced_name,
 				 named_name, named_name,
@@ -648,7 +655,7 @@ resource_widget_value (XlwMenuWidget mw, widget_value *val)
 	resourced_name = val->name;
 
       /* Parameterize the string. */
-      converted_name = parameterize_string(resourced_name, val->value);
+      converted_name = parameterize_string (resourced_name, val->value);
 
       /* nuke newline characters to prevent menubar screwups */
       for ( str = converted_name ; *str ; str++ )
@@ -673,14 +680,12 @@ resource_widget_value (XlwMenuWidget mw, widget_value *val)
 
 /* Unused */
 #if 0
-/*
- *    These two routines should be a seperate file..djw
- */
+/* These two routines should be a seperate file..djw */
 static char *
 xlw_create_localized_string (Widget w,
 			     char *name,
 			     char **args,
-			     unsigned nargs)
+			     unsigned int nargs)
 {
   char *string = NULL;
   char *arg = NULL;
@@ -705,11 +710,11 @@ static XmString
 xlw_create_localized_xmstring (Widget w,
 			       char *name,
 			       char **args,
-			       unsigned nargs)
+			       unsigned int nargs)
 {
   char *   string = xlw_create_localized_string (w, name, args, nargs);
   XmString xm_string = XmStringCreateLtoR (string, XmSTRING_DEFAULT_CHARSET);
-  XtFree(string);
+  XtFree (string);
   return xm_string;
 }
 #endif /* 0 */
@@ -737,7 +742,7 @@ resource_widget_value (XlwMenuWidget mw, widget_value *val)
       if (!resourced_name)
 	resourced_name = val->name;
 
-      complete_name = parameterize_string(resourced_name, val->value);
+      complete_name = parameterize_string (resourced_name, val->value);
 
       val->toolkit_data = complete_name;
       /* nuke newline characters to prevent menubar screwups */
@@ -753,18 +758,16 @@ resource_widget_value (XlwMenuWidget mw, widget_value *val)
 
 #endif /* !Motif */
 
-/*
- *    Code for drawing strings.
- */
+/* Code for drawing strings. */
 static void
-string_draw(XlwMenuWidget mw,
-	    Window window,
-	    int x, int y,
-	    GC gc,
+string_draw (XlwMenuWidget mw,
+	     Window window,
+	     int x, int y,
+	     GC gc,
 #ifdef NEED_MOTIF
-	    XmString string
+	     XmString string
 #else
-	    char *string
+	     char *string
 #endif
 )
 {
@@ -890,6 +893,9 @@ char *chars;
       }
   }
   x += string_draw_range (mw, window, x, y, gc, chars, s, i);
+#ifdef NEED_MOTIF
+  XtFree (chars);
+#endif
 }
 
 static void
@@ -904,16 +910,16 @@ binding_draw (XlwMenuWidget mw, Window w, int x, int y, GC gc, char *value)
 #endif
 }
 
-/*
- *    Low level code for drawing 3-D edges.
- */
+/* Low level code for drawing 3-D edges. */
 static void
 shadow_rectangle_draw (Display *dpy,
 		       Window window,
 		       GC top_gc,
 		       GC bottom_gc,
-		       int x, int y, unsigned width, unsigned height,
-		       unsigned thickness)
+		       int x, int y,
+		       unsigned int width,
+		       unsigned int height,
+		       unsigned int thickness)
 {
   XPoint points [4];
 
@@ -983,7 +989,9 @@ typedef enum e_shadow_type
 static void
 shadow_draw (XlwMenuWidget mw,
 	     Window window,
-	     int x, int y, unsigned width, unsigned height,
+	     int x, int y,
+	     unsigned int width,
+	     unsigned int height,
 	     shadow_type type)
 {
   Display *dpy = XtDisplay (mw);
@@ -1023,7 +1031,7 @@ shadow_draw (XlwMenuWidget mw,
 
   if (etched)
     {
-      unsigned half = thickness/2;
+      unsigned int half = thickness/2;
       shadow_rectangle_draw (dpy,
 			     window,
 			     top_gc,
@@ -1055,7 +1063,7 @@ static void
 arrow_decoration_draw (XlwMenuWidget mw,
 		       Window window,
 		       int x, int y,
-		       unsigned width,
+		       unsigned int width,
 		       Boolean raised)
 {
   Display *dpy = XtDisplay (mw);
@@ -1086,10 +1094,8 @@ arrow_decoration_draw (XlwMenuWidget mw,
       bottom_gc = mw->menu.shadow_bottom_gc;
     }
 
-  /*
-   *    Fill internal area, we do this first so that the borders
-   *    have a nice sharp edge.
-   */
+  /* Fill internal area.  We do this first so that the borders have a
+     nice sharp edge.  */
   points [0].x = x + thickness;
   points [0].y = y + thickness;
   points [1].x = x + length - thickness;
@@ -1148,7 +1154,7 @@ static void
 toggle_decoration_draw (XlwMenuWidget mw,
 			Window window,
 			int x, int y,
-			unsigned width,
+			unsigned int width,
 			Boolean set)
 {
   Display *dpy = XtDisplay (mw);
@@ -1161,9 +1167,7 @@ toggle_decoration_draw (XlwMenuWidget mw,
   else
     type = SHADOW_OUT;
 
-  /*
-   *    Fill internal area.
-   */
+  /* Fill internal area. */
   if (set)
     XFillRectangle (dpy,
 		    window,
@@ -1173,14 +1177,14 @@ toggle_decoration_draw (XlwMenuWidget mw,
 		    width - (2*thickness),
 		    width - (2*thickness));
 
-  shadow_draw(mw, window, x, y, width, width, type);
+  shadow_draw (mw, window, x, y, width, width, type);
 }
 
 static void
 radio_decoration_draw (XlwMenuWidget mw,
 		       Window window,
 		       int x, int y,
-		       unsigned width,
+		       unsigned int width,
 		       Boolean enabled)
 {
   Display *dpy = XtDisplay (mw);
@@ -1211,10 +1215,8 @@ radio_decoration_draw (XlwMenuWidget mw,
     }
 
 #if 1
-  /*
-   *    Draw the bottom first, just incase the regions overlap.
-   *    The top should cast the longer shadow.
-   */
+  /*  Draw the bottom first, just in case the regions overlap.
+      The top should cast the longer shadow. */
   points [0].x = x; /* left corner */
   points [0].y = y + half_width;
   points [1].x = x + half_width; /* bottom corner */
@@ -1259,10 +1261,8 @@ radio_decoration_draw (XlwMenuWidget mw,
 
   XFillPolygon (dpy, window, top_gc, points, 4, Convex, CoordModeOrigin);
 #else
-  /*
-   *    Draw the bottom first, just incase the regions overlap.
-   *    The top should cast the longer shadow.
-   */
+  /* Draw the bottom first, just in case the regions overlap.
+     The top should cast the longer shadow. */
   npoints = 0;
   points [npoints].x = x; /* left corner */
   points [npoints++].y = y + half_width;
@@ -1300,9 +1300,7 @@ radio_decoration_draw (XlwMenuWidget mw,
 #endif
 
 
-  /*
-   *    Fill internal area.
-   */
+  /* Fill internal area. */
   if (enabled)
     {
       points [0].x = x + thickness;
@@ -1327,19 +1325,18 @@ static void
 separator_decoration_draw (XlwMenuWidget mw,
 			   Window window,
 			   int x, int y,
-			   unsigned width,
+			   unsigned int width,
 			   Boolean vertical,
 			   shadow_type type)
 {
   Display *dpy = XtDisplay (mw);
   GC top_gc;
   GC bottom_gc;
-  unsigned offset = 0;
-  unsigned num_separators = 1;
-  unsigned top_line_thickness = 0;
-  unsigned bottom_line_thickness = 0;
+  unsigned int offset = 0;
+  unsigned int num_separators = 1;
+  unsigned int top_line_thickness = 0;
+  unsigned int bottom_line_thickness = 0;
   Boolean dashed = False;
-  int i;
 
   switch (type)
     {
@@ -1407,6 +1404,7 @@ separator_decoration_draw (XlwMenuWidget mw,
 
   while (num_separators--)
     {
+      unsigned int i;
       for (i = 0; i < top_line_thickness; i++)
 	XDrawLine (dpy, window, top_gc, x, y + i, x + width, y + i);
 
@@ -1450,7 +1448,8 @@ print_widget_value (widget_value *wv, int just_one, int depth)
 {
   char d [200];
   int i;
-  for (i = 0; i < depth; i++) d[i] = ' ';
+  for (i = 0; i < depth; i++)
+    d[i] = ' ';
   d[depth]=0;
   if (!wv)
     {
@@ -1501,7 +1500,7 @@ menu_item_type (widget_value *val)
   else
     {
 #if SLOPPY_TYPES
-      if (all_dashes_p(val->name))
+      if (all_dashes_p (val->name))
 	return SEPARATOR_TYPE;
       else if (val->name && val->name[0] == '\0') /* push right */
 	return PUSHRIGHT_TYPE;
@@ -1521,10 +1520,10 @@ static void
 label_button_size (XlwMenuWidget mw,
 		   widget_value *val,
 		   Boolean in_menubar,
-		   unsigned *toggle_width,
-		   unsigned *label_width,
-		   unsigned *bindings_width,
-		   unsigned *height)
+		   unsigned int *toggle_width,
+		   unsigned int *label_width,
+		   unsigned int *bindings_width,
+		   unsigned int *height)
 {
   *height = (mw->menu.font_ascent + mw->menu.font_descent +
 	     2 * mw->menu.vertical_margin +
@@ -1543,23 +1542,29 @@ label_button_draw (XlwMenuWidget mw,
 		   Boolean       highlighted,
 		   Window        window,
 		   int x, int y,
-		   unsigned width,
-		   unsigned height,
-		   unsigned label_offset,
-		   unsigned binding_tab)
+		   unsigned int width,
+		   unsigned int height,
+		   unsigned int label_offset,
+		   unsigned int binding_tab)
 {
   int y_offset = mw->menu.shadow_thickness + mw->menu.vertical_margin;
+  GC gc;
 
   if (!label_offset)
     label_offset = mw->menu.shadow_thickness + mw->menu.horizontal_margin;
 
-  /*
-   *    Draw the label string.
-   */
+  if (highlighted && (in_menubar || val->contents))
+    gc = mw->menu.highlight_gc;
+  else if (in_menubar || val->contents)
+    gc = mw->menu.foreground_gc;
+  else
+    gc = mw->menu.title_gc;
+
+  /*  Draw the label string. */
   string_draw_u (mw,
 	       window,
 	       x + label_offset, y + y_offset,
-	       mw->menu.foreground_gc,
+	       gc,
 	       resource_widget_value (mw, val));
 }
 
@@ -1567,10 +1572,10 @@ static void
 push_button_size (XlwMenuWidget mw,
 		  widget_value *val,
 		  Boolean in_menubar,
-		  unsigned *toggle_width,
-		  unsigned *label_width,
-		  unsigned *bindings_width,
-		  unsigned *height)
+		  unsigned int *toggle_width,
+		  unsigned int *label_width,
+		  unsigned int *bindings_width,
+		  unsigned int *height)
 {
   /* inherit */
   label_button_size (mw, val, in_menubar,
@@ -1583,7 +1588,7 @@ push_button_size (XlwMenuWidget mw,
       int w;
 #ifdef NEED_MOTIF
       XmString key = XmStringCreateLtoR (val->key, XmSTRING_DEFAULT_CHARSET);
-      w = string_width(mw, key);
+      w = string_width (mw, key);
       XmStringFree (key);
 #else
       char *key = val->key;
@@ -1600,22 +1605,28 @@ push_button_draw (XlwMenuWidget mw,
 		  Boolean       highlighted,
 		  Window        window,
 		  int x, int y,
-		  unsigned width, unsigned height,
-		  unsigned      label_offset,
-		  unsigned      binding_offset)
+		  unsigned int width,
+		  unsigned int height,
+		  unsigned int label_offset,
+		  unsigned int binding_offset)
 {
   int y_offset = mw->menu.shadow_thickness + mw->menu.vertical_margin;
   GC gc;
   shadow_type type;
   Boolean menu_pb = in_menubar && (menu_item_type (val) == BUTTON_TYPE);
 
-  /*
-   *    Draw the label string.
-   */
+  /* Draw the label string. */
   if (!label_offset)
     label_offset = mw->menu.shadow_thickness + mw->menu.horizontal_margin;
 
-  if (menu_pb)
+  if (highlighted)
+    {
+      if (val->enabled)
+	gc = mw->menu.highlight_gc;
+      else
+	gc = mw->menu.inactive_gc;
+    }
+  else if (menu_pb)
     {
       if (val->enabled)
 	gc = mw->menu.button_gc;
@@ -1634,16 +1645,15 @@ push_button_draw (XlwMenuWidget mw,
 	       window,
 	       x + label_offset, y + y_offset,
 	       gc,
-	       resource_widget_value(mw, val));
+	       resource_widget_value (mw, val));
 
-  /*
-   *    Draw the keybindings
-   */
+  /* Draw the keybindings */
   if (val->key)
     {
       if (!binding_offset)
 	{
-	  unsigned s_width = string_width (mw, resource_widget_value(mw, val));
+	  unsigned int s_width =
+	    string_width (mw, resource_widget_value (mw, val));
 	  binding_offset = label_offset + s_width +  mw->menu.shadow_thickness;
 	}
       binding_draw (mw, window,
@@ -1651,9 +1661,7 @@ push_button_draw (XlwMenuWidget mw,
 		    y + y_offset, gc, val->key);
     }
 
-  /*
-   *    Draw the shadow
-   */
+  /* Draw the shadow */
   if (menu_pb)
     {
       if (highlighted)
@@ -1675,8 +1683,7 @@ push_button_draw (XlwMenuWidget mw,
 static unsigned int
 arrow_decoration_height (XlwMenuWidget mw)
 {
-  unsigned int result =
-    (mw->menu.font_ascent + mw->menu.font_descent) / (unsigned int)2;
+  int result = (mw->menu.font_ascent + mw->menu.font_descent) / 2;
 
   result += 2 * mw->menu.shadow_thickness;
 
@@ -1690,10 +1697,10 @@ static void
 cascade_button_size (XlwMenuWidget mw,
 		     widget_value *val,
 		     Boolean in_menubar,
-		     unsigned *toggle_width,
-		     unsigned *label_width,
-		     unsigned *arrow_width,
-		     unsigned *height)
+		     unsigned int *toggle_width,
+		     unsigned int *label_width,
+		     unsigned int *arrow_width,
+		     unsigned int *height)
 {
   /* inherit */
   label_button_size (mw, val, in_menubar,
@@ -1702,7 +1709,7 @@ cascade_button_size (XlwMenuWidget mw,
   /* we have a pull aside arrow */
   if (!in_menubar)
     {
-      *arrow_width += arrow_decoration_height(mw) + mw->menu.column_spacing;
+      *arrow_width += arrow_decoration_height (mw) + mw->menu.column_spacing;
     }
 }
 
@@ -1713,33 +1720,31 @@ cascade_button_draw (XlwMenuWidget mw,
 		     Boolean       highlighted,
 		     Window        window,
 		     int x, int y,
-		     unsigned width, unsigned height,
-		     unsigned      label_offset,
-		     unsigned      binding_offset)
+		     unsigned int  width,
+		     unsigned int  height,
+		     unsigned int  label_offset,
+		     unsigned int  binding_offset)
 {
   shadow_type type;
 
-  /*
-   *    Draw the label string.
-   */
+  /* Draw the label string. */
   label_button_draw (mw, val, in_menubar, highlighted,
 		     window, x, y, width, height, label_offset,
 		     binding_offset);
 
-  /*
-   *    Draw the pull aside arrow
-   */
+  /* Draw the pull aside arrow */
   if (!in_menubar && val->contents)
     {
       int y_offset;
-      unsigned arrow_height = arrow_decoration_height (mw);
+      unsigned int arrow_height = arrow_decoration_height (mw);
 
       y_offset = mw->menu.shadow_thickness + mw->menu.vertical_margin +
 	(mw->menu.font_ascent+mw->menu.font_descent - arrow_height)/2;
 
       if (!binding_offset)
 	{
-	  unsigned s_width = string_width(mw, resource_widget_value (mw, val));
+	  unsigned int s_width =
+	    string_width (mw, resource_widget_value (mw, val));
 
 	  if (!label_offset)
 	    label_offset = mw->menu.shadow_thickness +
@@ -1756,28 +1761,31 @@ cascade_button_draw (XlwMenuWidget mw,
 			     highlighted);
     }
 
-  /*
-   *    Draw the shadow
-   */
+  /* Draw the shadow */
   if (highlighted)
     type = SHADOW_OUT;
   else
     type = SHADOW_BACKGROUND;
 
-  shadow_draw(mw, window, x, y, width, height, type);
+  shadow_draw (mw, window, x, y, width, height, type);
 }
 
-static unsigned
-toggle_decoration_height(XlwMenuWidget mw)
+static unsigned int
+toggle_decoration_height (XlwMenuWidget mw)
 {
-  unsigned rv;
+  int rv;
   if (mw->menu.indicator_size > 0)
     rv = mw->menu.indicator_size;
   else
     rv = mw->menu.font_ascent;
 
-  if (rv > (mw->menu.font_ascent+mw->menu.font_descent))
-    rv = mw->menu.font_ascent+mw->menu.font_descent;
+  if (rv > (mw->menu.font_ascent + mw->menu.font_descent))
+    rv = mw->menu.font_ascent + mw->menu.font_descent;
+
+  /* radio button can't be smaller than its border or a filling
+     error will occur. */
+  if (rv < 2 * mw->menu.shadow_thickness)
+    rv = 2 * mw->menu.shadow_thickness;
 
   return rv;
 }
@@ -1786,17 +1794,17 @@ static void
 toggle_button_size (XlwMenuWidget mw,
 		    widget_value *val,
 		    Boolean in_menubar,
-		    unsigned *toggle_width,
-		    unsigned *label_width,
-		    unsigned *bindings_width,
-		    unsigned *height)
+		    unsigned int *toggle_width,
+		    unsigned int *label_width,
+		    unsigned int *bindings_width,
+		    unsigned int *height)
 {
   /* inherit */
   push_button_size (mw, val, in_menubar,
 		    toggle_width, label_width, bindings_width,
 		    height);
   /* we have a toggle */
-  *toggle_width += toggle_decoration_height(mw) + mw->menu.column_spacing;
+  *toggle_width += toggle_decoration_height (mw) + mw->menu.column_spacing;
 }
 
 static void
@@ -1806,17 +1814,16 @@ toggle_button_draw (XlwMenuWidget mw,
 		    Boolean highlighted,
 		    Window        window,
 		    int x, int y,
-		    unsigned width, unsigned height,
-		    unsigned      label_tab,
-		    unsigned      binding_tab)
+		    unsigned int  width,
+		    unsigned int  height,
+		    unsigned int  label_tab,
+		    unsigned int  binding_tab)
 {
   int x_offset;
   int y_offset;
-  unsigned t_height = toggle_decoration_height(mw);
+  unsigned int t_height = toggle_decoration_height (mw);
 
-  /*
-   *    Draw a toggle.
-   */
+  /* Draw a toggle. */
   x_offset = mw->menu.shadow_thickness + mw->menu.horizontal_margin;
   y_offset = mw->menu.shadow_thickness + mw->menu.vertical_margin;
   y_offset += (mw->menu.font_ascent + mw->menu.font_descent - t_height)/2;
@@ -1824,17 +1831,15 @@ toggle_button_draw (XlwMenuWidget mw,
   toggle_decoration_draw (mw, window, x + x_offset, y + y_offset,
 			  t_height, val->selected);
 
-  /*
-   *    Draw the pushbutton parts.
-   */
+  /* Draw the pushbutton parts. */
   push_button_draw (mw, val, in_menubar, highlighted, window, x, y, width,
 		    height, label_tab, binding_tab);
 }
 
-static unsigned
-radio_decoration_height(XlwMenuWidget mw)
+static unsigned int
+radio_decoration_height (XlwMenuWidget mw)
 {
-  return toggle_decoration_height(mw);
+  return toggle_decoration_height (mw);
 }
 
 static void
@@ -1844,17 +1849,16 @@ radio_button_draw (XlwMenuWidget mw,
 		   Boolean       highlighted,
 		   Window        window,
 		   int x, int y,
-		   unsigned width, unsigned height,
-		   unsigned      label_tab,
-		   unsigned      binding_tab)
+		   unsigned int  width,
+		   unsigned int  height,
+		   unsigned int  label_tab,
+		   unsigned int  binding_tab)
 {
   int x_offset;
   int y_offset;
-  unsigned r_height = radio_decoration_height(mw);
+  unsigned int r_height = radio_decoration_height (mw);
 
-  /*
-   *    Draw a toggle.
-   */
+  /* Draw a toggle. */
   x_offset = mw->menu.shadow_thickness + mw->menu.horizontal_margin;
   y_offset = mw->menu.shadow_thickness + mw->menu.vertical_margin;
   y_offset += (mw->menu.font_ascent + mw->menu.font_descent - r_height)/2;
@@ -1862,9 +1866,7 @@ radio_button_draw (XlwMenuWidget mw,
   radio_decoration_draw (mw, window, x + x_offset, y + y_offset, r_height,
 			 val->selected);
 
-  /*
-   *    Draw the pushbutton parts.
-   */
+  /* Draw the pushbutton parts. */
   push_button_draw (mw, val, in_menubar, highlighted, window, x, y, width,
 		    height, label_tab, binding_tab);
 }
@@ -1895,11 +1897,10 @@ static struct _shadow_names
 static shadow_type
 separator_type (char *name)
 {
-  int i;
-
   if (name)
     {
-      for (i = 0; i < XtNumber(shadow_names); i++ )
+      int i;
+      for (i = 0; i < (int) (XtNumber (shadow_names)); i++ )
 	{
 	  if (strcmp (name, shadow_names[i].name) == 0)
 	    return shadow_names[i].type;
@@ -1908,11 +1909,11 @@ separator_type (char *name)
   return SHADOW_BACKGROUND;
 }
 
-static unsigned
+static unsigned int
 separator_decoration_height (XlwMenuWidget mw, widget_value *val)
 {
 
-  switch (separator_type(val->value))
+  switch (separator_type (val->value))
     {
     case SHADOW_NO_LINE:
     case SHADOW_SINGLE_LINE:
@@ -1937,10 +1938,10 @@ static void
 separator_size (XlwMenuWidget mw,
 		widget_value *val,
 		Boolean in_menubar,
-		unsigned *toggle_width,
-		unsigned *label_width,
-		unsigned *rest_width,
-		unsigned *height)
+		unsigned int *toggle_width,
+		unsigned int *label_width,
+		unsigned int *rest_width,
+		unsigned int *height)
 {
   *height = separator_decoration_height (mw, val);
   *label_width = 1;
@@ -1954,11 +1955,12 @@ separator_draw (XlwMenuWidget mw,
 		Boolean       highlighted,
 		Window        window,
 		int x, int y,
-		unsigned width, unsigned height,
-		unsigned      label_tab,
-		unsigned      binding_tab)
+		unsigned int  width,
+		unsigned int  height,
+		unsigned int  label_tab,
+		unsigned int  binding_tab)
 {
-  unsigned sep_width;
+  unsigned int sep_width;
 
   if (in_menubar)
     sep_width = height;
@@ -1978,11 +1980,10 @@ static void
 pushright_size (XlwMenuWidget mw,
 		widget_value *val,
 		Boolean in_menubar,
-		unsigned *toggle_width,
-		unsigned *label_width,
-		unsigned *rest_width,
-		unsigned *height
-)
+		unsigned int *toggle_width,
+		unsigned int *label_width,
+		unsigned int *rest_width,
+		unsigned int *height)
 {
   *height = *label_width = *toggle_width = *rest_width = 0;
 }
@@ -1991,19 +1992,18 @@ static void
 size_menu_item (XlwMenuWidget mw,
 		widget_value *val,
 		int horizontal,
-		unsigned *toggle_width,
-		unsigned *label_width,
-		unsigned *rest_width,
-		unsigned *height
-)
+		unsigned int *toggle_width,
+		unsigned int *label_width,
+		unsigned int *rest_width,
+		unsigned int *height)
 {
   void (*function_ptr) (XlwMenuWidget _mw,
 			widget_value *_val,
 			Boolean _in_menubar,
-			unsigned *_toggle_width,
-			unsigned *_label_width,
-			unsigned *_rest_width,
-			unsigned *_height);
+			unsigned int *_toggle_width,
+			unsigned int *_label_width,
+			unsigned int *_rest_width,
+			unsigned int *_height);
 
   switch (menu_item_type (val))
     {
@@ -2051,22 +2051,23 @@ display_menu_item (XlwMenuWidget mw,
 
   int x = where->x /* + mw->menu.shadow_thickness */ ;
   int y = where->y /* + mw->menu.shadow_thickness */ ;
-  unsigned toggle_width;
-  unsigned label_width;
-  unsigned binding_width;
-  unsigned width;
-  unsigned height;
-  unsigned label_tab;
-  unsigned binding_tab;
+  unsigned int toggle_width;
+  unsigned int label_width;
+  unsigned int binding_width;
+  unsigned int width;
+  unsigned int height;
+  unsigned int label_tab;
+  unsigned int binding_tab;
   void (*function_ptr) (XlwMenuWidget _mw,
 			widget_value *_val,
 			Boolean _in_menubar,
 			Boolean _highlighted,
 			Window        _window,
 			int _x, int _y,
-			unsigned _width, unsigned _height,
-			unsigned      _label_tab,
-			unsigned      _binding_tab);
+			unsigned int _width,
+			unsigned int _height,
+			unsigned int _label_tab,
+			unsigned int  _binding_tab);
 
   size_menu_item (mw, val, horizontal,
 		  &toggle_width, &label_width, &binding_width, &height);
@@ -2131,14 +2132,14 @@ display_menu_item (XlwMenuWidget mw,
 static void
 size_menu (XlwMenuWidget mw, int level)
 {
-  unsigned      toggle_width;
-  unsigned	label_width;
-  unsigned	rest_width;
-  unsigned	height;
-  unsigned	max_toggle_width = 0;
-  unsigned	max_label_width  = 0;
-  unsigned	max_rest_width   = 0;
-  unsigned	max_height = 0;
+  unsigned int	toggle_width;
+  unsigned int	label_width;
+  unsigned int	rest_width;
+  unsigned int	height;
+  unsigned int	max_toggle_width = 0;
+  unsigned int	max_label_width  = 0;
+  unsigned int	max_rest_width   = 0;
+  unsigned int	max_height = 0;
   int		horizontal_p = mw->menu.horizontal && (level == 0);
   widget_value*	val;
   window_state*	ws;
@@ -2215,21 +2216,6 @@ display_menu (XlwMenuWidget mw, int level, Boolean just_compute_p,
   puts("===================================================================");
   print_widget_value (following_item, 1, 0);
 #endif
-  if (following_item
-      && following_item->type == CASCADE_TYPE
-      && following_item->contents
-      && following_item->contents->type == INCREMENTAL_TYPE)
-    {
-      /* okay, we're now doing a lisp callback to incrementally generate
-	 more of the menu. */
-      XtCallCallbackList ((Widget)mw,
-			  mw->menu.open,
-			  (XtPointer)following_item->contents);
-#if SLOPPY_TYPES == 1
-  puts("==== NEW ==== NEW ==== NEW ==== NEW ==== NEW ==== NEW ==== NEW ====");
-  print_widget_value(following_item, 1, 0);
-#endif
-    }
 
   if (hit)
     *hit_return = NULL;
@@ -2327,16 +2313,31 @@ make_windows_if_needed (XlwMenuWidget mw, int n)
   int i;
   int start_at;
   XSetWindowAttributes xswa;
+  Widget p;
   int mask;
-#define ROOT_PARENT
-#ifdef ROOT_PARENT
-  Window root = RootWindowOfScreen (DefaultScreenOfDisplay (XtDisplay (mw)));
-#endif
+  int depth;
+  Visual *visual;
   window_state *windows;
+  Window root;
 
   if (mw->menu.windows_length >= n)
     return;
 
+  root = RootWindowOfScreen (XtScreen(mw));
+  /* grab the visual and depth from the nearest shell ancestor */
+  visual = CopyFromParent;
+  depth = CopyFromParent;
+  p = XtParent(mw);
+  while (visual == CopyFromParent && p)
+    {
+      if (XtIsShell(p))
+	{
+	  visual = ((ShellWidget)p)->shell.visual;
+	  depth = p->core.depth;
+	}
+      p = XtParent(p);
+    }
+
   xswa.save_under = True;
   xswa.override_redirect = True;
   xswa.background_pixel = mw->core.background_pixel;
@@ -2344,8 +2345,9 @@ make_windows_if_needed (XlwMenuWidget mw, int n)
   xswa.event_mask = (ExposureMask | ButtonMotionMask
 		     | ButtonReleaseMask | ButtonPressMask);
   xswa.cursor = mw->menu.cursor_shape;
+  xswa.colormap = mw->core.colormap;
   mask = CWSaveUnder | CWOverrideRedirect | CWBackPixel | CWBorderPixel
-    | CWEventMask | CWCursor;
+    | CWEventMask | CWCursor | CWColormap;
 
   if (mw->menu.use_backing_store)
     {
@@ -2378,15 +2380,9 @@ make_windows_if_needed (XlwMenuWidget mw, int n)
      windows [i].height = 1;
      windows [i].window =
        XCreateWindow (XtDisplay (mw),
-#ifdef ROOT_PARENT
 		      root,
-#else
-		      ((i >  0)
-		       ? windows[0].window
-		       : XtWindow (XtParent (mw))),
-#endif
 		      0, 0, 1, 1,
-		      0, 0, CopyFromParent, CopyFromParent, mask, &xswa);
+		      0, depth, CopyFromParent, visual, mask, &xswa);
   }
 }
 
@@ -2477,11 +2473,7 @@ remap_menubar (XlwMenuWidget mw)
 
   /* Memorize the previously selected item to be able to refresh it */
   old_selection = last_same + 1 < old_depth ? old_stack [last_same + 1] : NULL;
-  if (old_selection && !old_selection->enabled)
-    old_selection = NULL;
   new_selection = last_same + 1 < new_depth ? new_stack [last_same + 1] : NULL;
-  if (new_selection && !new_selection->enabled)
-    new_selection = NULL;
 
   /* updates old_state from new_state.  It has to be done now because
      display_menu (called below) uses the old_stack to know what to display. */
@@ -2511,6 +2503,18 @@ remap_menubar (XlwMenuWidget mw)
       /* take into account the slab around the new menu */
       ws->y -= mw->menu.shadow_thickness;
 
+      {
+	widget_value *val = mw->menu.old_stack [i];
+	if (val->contents->type == INCREMENTAL_TYPE)
+	{
+	  /* okay, we're now doing a lisp callback to incrementally generate
+	     more of the menu. */
+	  XtCallCallbackList ((Widget)mw,
+			      mw->menu.open,
+			      (XtPointer)val->contents);
+	}
+      }
+
       size_menu (mw, i);
 
       fit_to_screen (mw, ws, previous_ws, mw->menu.horizontal && i == 1);
@@ -2609,11 +2613,11 @@ make_drawing_gcs (XlwMenuWidget mw)
 
   xgcv.foreground = mw->core.background_pixel;
   xgcv.background = mw->menu.foreground;
-  mw->menu.background_gc = XtGetGC ((Widget)mw, flags, &xgcv);
+  mw->menu.background_gc = XtGetGC ((Widget) mw, flags, &xgcv);
 
   xgcv.foreground = mw->menu.foreground;
   xgcv.background = mw->core.background_pixel;
-  mw->menu.foreground_gc = XtGetGC ((Widget)mw, flags, &xgcv);
+  mw->menu.foreground_gc = XtGetGC ((Widget) mw, flags, &xgcv);
 
   if (mw->menu.select_color != (Pixel)-1)
     {
@@ -2629,12 +2633,12 @@ make_drawing_gcs (XlwMenuWidget mw)
       else
 	{ /* color */
 	  XColor xcolor;
-	  Colormap cmap = DefaultColormapOfScreen (XtScreen ((Widget) mw));
+	  Colormap cmap = mw->core.colormap;
 	  xcolor.pixel = mw->core.background_pixel;
 	  XQueryColor (dpy, cmap, &xcolor);
-	  xcolor.red   *= 0.85;
-	  xcolor.green *= 0.85;
-	  xcolor.blue  *= 0.85;
+	  xcolor.red   = (xcolor.red   * 17) / 20;
+	  xcolor.green = (xcolor.green * 17) / 20;
+	  xcolor.blue  = (xcolor.blue  * 17) / 20;
 	  if (allocate_nearest_color (dpy, cmap, &xcolor))
 	    xgcv.foreground = xcolor.pixel;
 	}
@@ -2650,6 +2654,14 @@ make_drawing_gcs (XlwMenuWidget mw)
 				  (flags | GCFillStyle | GCStipple),
 				  &xgcv);
 
+  xgcv.foreground = mw->menu.highlight_foreground;
+  xgcv.background = mw->core.background_pixel;
+  mw->menu.highlight_gc = XtGetGC ((Widget)mw, flags, &xgcv);
+
+  xgcv.foreground = mw->menu.title_foreground;
+  xgcv.background = mw->core.background_pixel;
+  mw->menu.title_gc = XtGetGC ((Widget)mw, flags, &xgcv);
+
   xgcv.foreground = mw->menu.button_foreground;
   xgcv.background = mw->core.background_pixel;
   mw->menu.button_gc = XtGetGC ((Widget)mw, flags, &xgcv);
@@ -2666,6 +2678,8 @@ release_drawing_gcs (XlwMenuWidget mw)
 {
   XtReleaseGC ((Widget) mw, mw->menu.foreground_gc);
   XtReleaseGC ((Widget) mw, mw->menu.button_gc);
+  XtReleaseGC ((Widget) mw, mw->menu.highlight_gc);
+  XtReleaseGC ((Widget) mw, mw->menu.title_gc);
   XtReleaseGC ((Widget) mw, mw->menu.inactive_gc);
   XtReleaseGC ((Widget) mw, mw->menu.inactive_button_gc);
   XtReleaseGC ((Widget) mw, mw->menu.background_gc);
@@ -2673,6 +2687,8 @@ release_drawing_gcs (XlwMenuWidget mw)
   /* let's get some segvs if we try to use these... */
   mw->menu.foreground_gc      = (GC) -1;
   mw->menu.button_gc          = (GC) -1;
+  mw->menu.highlight_gc       = (GC) -1;
+  mw->menu.title_gc           = (GC) -1;
   mw->menu.inactive_gc        = (GC) -1;
   mw->menu.inactive_button_gc = (GC) -1;
   mw->menu.background_gc      = (GC) -1;
@@ -2688,13 +2704,13 @@ make_shadow_gcs (XlwMenuWidget mw)
   XGCValues xgcv;
   unsigned long pm = 0;
   Display *dpy = XtDisplay ((Widget) mw);
-  Colormap cmap = DefaultColormapOfScreen (XtScreen ((Widget) mw));
+  Colormap cmap = mw->core.colormap;
   XColor topc, botc;
   int top_frobbed = 0, bottom_frobbed = 0;
 
-  if (mw->menu.top_shadow_color == -1)
+  if (mw->menu.top_shadow_color == (Pixel) (-1))
       mw->menu.top_shadow_color = mw->core.background_pixel;
-  if (mw->menu.bottom_shadow_color == -1)
+  if (mw->menu.bottom_shadow_color == (Pixel) (-1))
       mw->menu.bottom_shadow_color = mw->menu.foreground;
 
   if (mw->menu.top_shadow_color == mw->core.background_pixel ||
@@ -2732,9 +2748,9 @@ make_shadow_gcs (XlwMenuWidget mw)
     {
       botc.pixel = mw->core.background_pixel;
       XQueryColor (dpy, cmap, &botc);
-      botc.red   *= 0.6;
-      botc.green *= 0.6;
-      botc.blue  *= 0.6;
+      botc.red   = (botc.red   * 3) / 5;
+      botc.green = (botc.green * 3) / 5;
+      botc.blue  = (botc.blue  * 3) / 5;
       if (allocate_nearest_color (dpy, cmap, &botc))
 	{
 	  if (botc.pixel == mw->core.background_pixel)
@@ -2792,13 +2808,31 @@ make_shadow_gcs (XlwMenuWidget mw)
   xgcv.fill_style = FillOpaqueStippled;
   xgcv.foreground = mw->menu.top_shadow_color;
   xgcv.background = mw->core.background_pixel;
+/*  xgcv.stipple = mw->menu.top_shadow_pixmap; gtb */
+#ifdef NEED_MOTIF
+  if (mw->menu.top_shadow_pixmap &&
+      mw->menu.top_shadow_pixmap != XmUNSPECIFIED_PIXMAP)
+     xgcv.stipple = mw->menu.top_shadow_pixmap;
+  else
+     xgcv.stipple = 0;
+#else
   xgcv.stipple = mw->menu.top_shadow_pixmap;
+#endif /* NEED_MOTIF */
   pm = (xgcv.stipple ? GCStipple|GCFillStyle : 0);
   mw->menu.shadow_top_gc =
     XtGetGC((Widget)mw, GCForeground|GCBackground|pm, &xgcv);
 
   xgcv.foreground = mw->menu.bottom_shadow_color;
+/*  xgcv.stipple = mw->menu.bottom_shadow_pixmap; gtb */
+#ifdef NEED_MOTIF
+  if (mw->menu.top_shadow_pixmap &&
+      mw->menu.top_shadow_pixmap != XmUNSPECIFIED_PIXMAP)
+     xgcv.stipple = mw->menu.bottom_shadow_pixmap;
+  else
+     xgcv.stipple = 0;
+#else
   xgcv.stipple = mw->menu.bottom_shadow_pixmap;
+#endif /* NEED_MOTIF */
   pm = (xgcv.stipple ? GCStipple|GCFillStyle : 0);
   mw->menu.shadow_bottom_gc =
     XtGetGC ((Widget)mw, GCForeground|GCBackground|pm, &xgcv);
@@ -3038,7 +3072,7 @@ XlwMenuInitialize (Widget request, Widget new, ArgList args,
 static void
 XlwMenuClassInitialize (void)
 {
-  Initialize_massaged_resource_char();
+  initialize_massaged_resource_char();
 }
 
 static void
@@ -3246,7 +3280,7 @@ handle_motion_event (XlwMenuWidget mw, XMotionEvent *ev,
 {
   int x = ev->x_root;
   int y = ev->y_root;
-  int state = ev->state;
+  unsigned int state = ev->state;
   XMotionEvent *event= ev, dummy;
 
   /* allow motion events to be generated again */
