@@ -79,7 +79,8 @@ Lisp_Object Vstandard_category_table;
    4) case tables
    5) keyboard-translate-table?
 
-   We provide an abstract type to generalize the Emacs vectors and Mule
+   We provide an
+   abstract type to generalize the Emacs vectors and Mule
    vectors-of-vectors goo.
    */
 
@@ -89,15 +90,6 @@ Lisp_Object Vstandard_category_table;
 
 #ifdef MULE
 
-static Lisp_Object mark_char_table_entry (Lisp_Object, void (*) (Lisp_Object));
-static int char_table_entry_equal (Lisp_Object, Lisp_Object, int depth);
-static unsigned long char_table_entry_hash (Lisp_Object obj, int depth);
-DEFINE_LRECORD_IMPLEMENTATION ("char-table-entry", char_table_entry,
-                               mark_char_table_entry, internal_object_printer,
-			       0, char_table_entry_equal,
-			       char_table_entry_hash,
-			       struct Lisp_Char_Table_Entry);
-
 static Lisp_Object
 mark_char_table_entry (Lisp_Object obj, void (*markobj) (Lisp_Object))
 {
@@ -133,17 +125,13 @@ char_table_entry_hash (Lisp_Object obj, int depth)
   return internal_array_hash (cte->level2, 96, depth);
 }
 
+DEFINE_LRECORD_IMPLEMENTATION ("char-table-entry", char_table_entry,
+                               mark_char_table_entry, internal_object_printer,
+			       0, char_table_entry_equal,
+			       char_table_entry_hash,
+			       struct Lisp_Char_Table_Entry);
 #endif /* MULE */
 
-static Lisp_Object mark_char_table (Lisp_Object, void (*) (Lisp_Object));
-static void print_char_table (Lisp_Object, Lisp_Object, int);
-static int char_table_equal (Lisp_Object, Lisp_Object, int depth);
-static unsigned long char_table_hash (Lisp_Object obj, int depth);
-DEFINE_LRECORD_IMPLEMENTATION ("char-table", char_table,
-                               mark_char_table, print_char_table, 0,
-			       char_table_equal, char_table_hash,
-			       struct Lisp_Char_Table);
-
 static Lisp_Object
 mark_char_table (Lisp_Object obj, void (*markobj) (Lisp_Object))
 {
@@ -430,6 +418,11 @@ char_table_hash (Lisp_Object obj, int depth)
   return hashval;
 }
 
+DEFINE_LRECORD_IMPLEMENTATION ("char-table", char_table,
+                               mark_char_table, print_char_table, 0,
+			       char_table_equal, char_table_hash,
+			       struct Lisp_Char_Table);
+
 DEFUN ("char-table-p", Fchar_table_p, 1, 1, 0, /*
 Return non-nil if OBJECT is a char table.
 
@@ -559,13 +552,15 @@ Reset a char table to its default state.
   switch (ct->type)
     {
     case CHAR_TABLE_TYPE_CHAR:
+      fill_char_table (ct, make_char (0));
+      break;
     case CHAR_TABLE_TYPE_DISPLAY:
     case CHAR_TABLE_TYPE_GENERIC:
 #ifdef MULE
     case CHAR_TABLE_TYPE_CATEGORY:
+#endif /* MULE */
       fill_char_table (ct, Qnil);
       break;
-#endif /* MULE */
 
     case CHAR_TABLE_TYPE_SYNTAX:
       fill_char_table (ct, make_int (Sinherit));
@@ -579,14 +574,14 @@ Reset a char table to its default state.
 }
 
 DEFUN ("make-char-table", Fmake_char_table, 1, 1, 0, /*
-Make a new, empty char table of type TYPE.
+Return a new, empty char table of type TYPE.
 Currently recognized types are 'char, 'category, 'display, 'generic,
 and 'syntax.  See `valid-char-table-type-p'.
 */
        (type))
 {
   struct Lisp_Char_Table *ct;
-  Lisp_Object obj = Qnil;
+  Lisp_Object obj;
   enum char_table_type ty = symbol_to_char_table_type (type);
 
   ct = alloc_lcrecord_type (struct Lisp_Char_Table, lrecord_char_table);
@@ -594,8 +589,8 @@ and 'syntax.  See `valid-char-table-type-p'.
   if (ty == CHAR_TABLE_TYPE_SYNTAX)
     {
       ct->mirror_table = Fmake_char_table (Qgeneric);
-      fill_char_table (XCHAR_TABLE (ct->mirror_table), 
-                       make_int (Spunct)); 
+      fill_char_table (XCHAR_TABLE (ct->mirror_table),
+                       make_int (Spunct));
     }
   else
     ct->mirror_table = Qnil;
@@ -615,14 +610,15 @@ and 'syntax.  See `valid-char-table-type-p'.
 static Lisp_Object
 make_char_table_entry (Lisp_Object initval)
 {
-  struct Lisp_Char_Table_Entry *cte;
-  Lisp_Object obj = Qnil;
+  Lisp_Object obj;
   int i;
+  struct Lisp_Char_Table_Entry *cte =
+    alloc_lcrecord_type (struct Lisp_Char_Table_Entry,
+			 lrecord_char_table_entry);
 
-  cte = alloc_lcrecord_type (struct Lisp_Char_Table_Entry,
-			     lrecord_char_table_entry);
   for (i = 0; i < 96; i++)
     cte->level2[i] = initval;
+
   XSETCHAR_TABLE_ENTRY (obj, cte);
   return obj;
 }
@@ -631,7 +627,7 @@ static Lisp_Object
 copy_char_table_entry (Lisp_Object entry)
 {
   struct Lisp_Char_Table_Entry *cte = XCHAR_TABLE_ENTRY (entry);
-  Lisp_Object obj = Qnil;
+  Lisp_Object obj;
   int i;
   struct Lisp_Char_Table_Entry *ctenew =
     alloc_lcrecord_type (struct Lisp_Char_Table_Entry,
@@ -660,7 +656,7 @@ as OLD-TABLE.  The values will not themselves be copied.
        (old_table))
 {
   struct Lisp_Char_Table *ct, *ctnew;
-  Lisp_Object obj = Qnil;
+  Lisp_Object obj;
   int i;
 
   CHECK_CHAR_TABLE (old_table);
@@ -781,7 +777,7 @@ get_non_ascii_char_table_value (struct Lisp_Char_Table *ct, int leading_byte,
 
 #endif /* MULE */
 
-static Lisp_Object
+Lisp_Object
 get_char_table (Emchar ch, struct Lisp_Char_Table *ct)
 {
 #ifdef MULE
@@ -993,6 +989,10 @@ canonicalize_char_table_value (Lisp_Object value, enum char_table_type type)
 	  CHECK_CHAR_COERCE_INT (cdr);
 	  return Fcons (car, cdr);
 	}
+      break;
+    case CHAR_TABLE_TYPE_CHAR:
+      CHECK_CHAR_COERCE_INT (value);
+      break;
     default:
       break;
     }
@@ -1601,7 +1601,8 @@ check_category_char (Emchar ch, Lisp_Object table,
 #endif
   ctbl = XCHAR_TABLE (table);
   temp = get_char_table (ch, ctbl);
-  if (EQ (temp, Qnil)) return not;
+  if (NILP (temp))
+    return not;
 
   designator -= ' ';
   return bit_vector_bit (XBIT_VECTOR (temp), designator) ? !not : not;
@@ -1610,8 +1611,8 @@ check_category_char (Emchar ch, Lisp_Object table,
 DEFUN ("check-category-at", Fcheck_category_at, 2, 4, 0, /*
 Return t if category of a character at POS includes DESIGNATOR,
 else return nil. Optional third arg specifies which buffer
-(defaulting to current), and fourth specifies the CATEGORY-TABLE,
-(defaulting to the buffer's category table).
+\(defaulting to current), and fourth specifies the CATEGORY-TABLE,
+\(defaulting to the buffer's category table).
 */
        (pos, designator, buffer, category_table))
 {
@@ -1622,7 +1623,7 @@ else return nil. Optional third arg specifies which buffer
 
   CHECK_INT (pos);
   CHECK_CATEGORY_DESIGNATOR (designator);
-  des = XREALINT (designator);
+  des = XCHAR (designator);
   ctbl = check_category_table (category_table, Vstandard_category_table);
   ch = BUF_FETCH_CHAR (buf, XINT (pos));
   return check_category_char (ch, ctbl, des, 0) ? Qt : Qnil;
@@ -1640,7 +1641,7 @@ which defaults to the system default table.
   unsigned int des;
 
   CHECK_CATEGORY_DESIGNATOR (designator);
-  des = XREALINT (designator);
+  des = XCHAR (designator);
   CHECK_CHAR (chr);
   ch = XCHAR (chr);
   ctbl = check_category_table (category_table, Vstandard_category_table);
