@@ -28,6 +28,9 @@ Boston, MA 02111-1307, USA.  */
 
 #include <config.h>
 #include "lisp.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include "buffer.h"
 #include "commands.h"
@@ -36,6 +39,7 @@ Boston, MA 02111-1307, USA.  */
 #include "frame.h"
 #include "insdel.h"
 #include "window.h"
+#include "line-number.h"
 
 #include "systime.h"
 #include "sysdep.h"
@@ -52,16 +56,22 @@ Lisp_Object Vsystem_name;	/* #### - I don't see why this should be */
 				   real values upon the first call to the
 				   functions that generate them. --stig */
 Lisp_Object Vuser_real_login_name; /* login name of current user ID */
-Lisp_Object Vuser_full_name;	/* full name of current user */
 Lisp_Object Vuser_login_name;	/* user name from LOGNAME or USER.  */
 #endif
 
-extern char *get_system_name (void);
+/* It's useful to be able to set this as user customization, so we'll
+   keep it. */
+Lisp_Object Vuser_full_name;
+EXFUN (Fuser_full_name, 1);
+
+char *get_system_name (void);
 
 Lisp_Object Qformat;
 
 Lisp_Object Qpoint, Qmark, Qregion_beginning, Qregion_end;
 
+Lisp_Object Quser_files_and_directories;
+
 /* This holds the value of `environ' produced by the previous
    call to Fset_time_zone_rule, or 0 if Fset_time_zone_rule
    has never been called.  */
@@ -71,101 +81,24 @@ void
 init_editfns (void)
 {
 /* Only used in removed code below. */
-#if 0
-  char *user_name;
-  Bufbyte *p, *q;
-  struct passwd *pw;	/* password entry for the current user */
-  Lisp_Object tem;
-#endif
+  char *p;
 
   environbuf = 0;
 
   /* Set up system_name even when dumping.  */
   init_system_name ();
 
-#if 0				/* this is now dynamic */
-  /* don't lose utterly if someone uses these during loadup. */
-  Vuser_real_login_name = Qnil;
-  Vuser_login_name = Qnil;
-  Vuser_full_name = Qnil;
-
 #ifndef CANNOT_DUMP
-  /* Don't bother with this on initial start when just dumping out */
   if (!initialized)
     return;
-#endif /* not CANNOT_DUMP */
-
-  pw = (struct passwd *) getpwuid (getuid ());
-#ifdef MSDOS
-  /* We let the real user name default to "root" because that's quite
-     accurate on MSDOG and because it lets Emacs find the init file.
-     (The DVX libraries override the Djgpp libraries here.)  */
-  Vuser_real_login_name = build_string (pw ? pw->pw_name : "root");
-#else
-  Vuser_real_login_name = build_string (pw ? pw->pw_name : "unknown");
 #endif
 
-  /* Get the effective user name, by consulting environment variables,
-     or the effective uid if those are unset.  */
-  user_name = getenv ("LOGNAME");
-  if (!user_name)
-#ifdef WINDOWSNT
-    user_name = (char *) getenv ("USERNAME"); /* it's USERNAME on NT */
-#else  /* WINDOWSNT */
-    user_name = (char *) getenv ("USER");
-#endif /* WINDOWSNT */
-  if (!user_name)
-    {
-      /* #### - do we really want the EFFECTIVE uid here?  Are these flipped? */
-      /* I ask because LOGNAME and USER vars WILL NOT MATCH the euid.  --Stig */
-      pw = (struct passwd *) getpwuid (geteuid ());
-      user_name = (char *) (pw ? pw->pw_name : "unknown");
-    }
-  Vuser_login_name = build_string (user_name);
-
-  /* If the user name claimed in the environment vars differs from
-     the real uid, use the claimed name to find the full name.  */
-  tem = Fstring_equal (Vuser_login_name, Vuser_real_login_name);
-  if (NILP (tem))
-    {
-      /* Jamie reports that IRIX gets wedged by SIGIO/SIGALARM occurring
-	 in select(), called from getpwnam(). */
-      slow_down_interrupts ();
-      pw = (struct passwd *)
-	getpwnam ((char *) XSTRING_DATA (Vuser_login_name));
-      speed_up_interrupts ();
-    }
-
-  p = (Bufbyte *) ((pw) ? USER_FULL_NAME : "unknown"); /* don't gettext here */
-  q = (Bufbyte *) strchr ((char *) p, ',');
-  Vuser_full_name = make_ext_string (p, (q ? q - p : strlen ((char *) p)),
-				     FORMAT_OS);
-
-#ifdef AMPERSAND_FULL_NAME
-  p = XSTRING_DATA (Vuser_full_name);
-  q = (Bufbyte *) strchr ((char *) p, '&');
-  /* Substitute the login name for the &, upcasing the first character.  */
-  if (q)
-    {
-      char *r = (char *)
-	alloca (strlen ((char *) p) + XSTRING_LENGTH (Vuser_login_name) + 1);
-      Charcount fullname_off = bytecount_to_charcount (p,  q - p);
-      memcpy (r, p, q - p);
-      r[q - p] = 0;
-      strcat (r, (char *) XSTRING_DATA (Vuser_login_name));
-      strcat (r, q + 1);
-      Vuser_full_name = build_string (r);
-      set_string_char (XSTRING (Vuser_full_name), fullname_off,
-		       UPCASE (current_buffer,
-			       string_char (XSTRING (Vuser_full_name),
-					    fullname_off)));
-    }
-#endif /* AMPERSAND_FULL_NAME */
-
-  p = (Bufbyte *) getenv ("NAME");
-  if (p)
-    Vuser_full_name = build_string ((char *) p);
-#endif /* 0 */
+  if ((p = getenv ("NAME")))
+    /* I don't think it's the right thing to do the ampersand
+       modification on NAME.  Not that it matters anymore...  -hniksic */
+    Vuser_full_name = build_ext_string (p, FORMAT_OS);
+  else
+    Vuser_full_name = Fuser_full_name (Qnil);
 }
 
 DEFUN ("char-to-string", Fchar_to_string, 1, 1, 0, /*
@@ -214,8 +147,7 @@ An empty string will return the constant `nil'.
 static Lisp_Object
 buildmark (Bufpos val, Lisp_Object buffer)
 {
-  Lisp_Object mark;
-  mark = Fmake_marker ();
+  Lisp_Object mark = Fmake_marker ();
   Fset_marker (mark, make_int (val), buffer);
   return mark;
 }
@@ -244,7 +176,8 @@ If BUFFER is nil, the current buffer is assumed.
   struct buffer *b = decode_buffer (buffer, 1);
   if (NILP (dont_copy_p))
     return Fcopy_marker (b->point_marker, Qnil);
-  return b->point_marker;
+  else
+    return b->point_marker;
 }
 
 /* The following two functions end up being identical but it's
@@ -253,23 +186,17 @@ If BUFFER is nil, the current buffer is assumed.
 Bufpos
 bufpos_clip_to_bounds (Bufpos lower, Bufpos num, Bufpos upper)
 {
-  if (num < lower)
-    return lower;
-  else if (num > upper)
-    return upper;
-  else
-    return num;
+  return (num < lower ? lower :
+	  num > upper ? upper :
+	  num);
 }
 
 Bytind
 bytind_clip_to_bounds (Bytind lower, Bytind num, Bytind upper)
 {
-  if (num < lower)
-    return lower;
-  else if (num > upper)
-    return upper;
-  else
-    return num;
+  return (num < lower ? lower :
+	  num > upper ? upper :
+	  num);
 }
 
 /*
@@ -393,86 +320,69 @@ If BUFFER is nil, the current buffer is assumed.
 }
 
 
-/* The saved object looks like this:
+/* The saved object is a cons:
 
-   (COPY-OF-POINT-MARKER . (COPY-OF-MARK . VISIBLE-P))
+   (COPY-OF-POINT-MARKER . COPY-OF-MARK)
 
-   where
-
-   VISIBLE-P is t if `(eq (current-buffer) (window-buffer (selected-window)))'
-   but is not actually used any more.
- */
+   We used to have another cons for a VISIBLE-P element, which was t
+   if `(eq (current-buffer) (window-buffer (selected-window)))' but it
+   was unused for a long time, so I removed it.  --hniksic */
 Lisp_Object
 save_excursion_save (void)
 {
   struct buffer *b;
-  int visible;
-  Lisp_Object tem;
 
-  if (preparing_for_armageddon)
-    return Qnil;
-  else
-    {
-      b = current_buffer;
-      visible = (XBUFFER (XWINDOW (Fselected_window (Qnil))->buffer) == b);
-      tem = ((visible) ? Qt : Qnil);
-    }
+  /* #### Huh?  --hniksic */
+  /*if (preparing_for_armageddon) return Qnil;*/
 
 #ifdef ERROR_CHECK_BUFPOS
   assert (XINT (Fpoint (Qnil)) ==
 	  XINT (Fmarker_position (Fpoint_marker (Qt, Qnil))));
 #endif
 
-#if 0 /* FSFmacs */
-  tem = Fcons (tem, b->mark_active);
-#endif
+  b = current_buffer;
 
-  return noseeum_cons (noseeum_copy_marker (Fpoint_marker (Qt, Qnil), Qnil),
-		       noseeum_cons (noseeum_copy_marker (b->mark, Qnil),
-				     tem));
+  return noseeum_cons (noseeum_copy_marker (b->point_marker, Qnil),
+		       noseeum_copy_marker (b->mark, Qnil));
 }
 
 Lisp_Object
 save_excursion_restore (Lisp_Object info)
 {
-  Lisp_Object tem;
-  int visible;
-  struct gcpro gcpro1, gcpro2;
+  Lisp_Object buffer = Fmarker_buffer (XCAR (info));
 
-  tem = Fmarker_buffer (Fcar (info));
-  /* If buffer being returned to is now deleted, avoid error */
-  /* Otherwise could get error here while unwinding to top level
-     and crash */
-  /* In that case, Fmarker_buffer returns nil now.  */
-  if (NILP (tem))
-    return Qnil;
-  /* Need gcpro in case Lisp hooks get run */
-  GCPRO2 (info, tem);
-  Fset_buffer (tem);
-  tem = Fcar (info);
-  Fgoto_char (tem, Fcurrent_buffer ());
-  tem = Fcar (Fcdr (info));
-  Fset_marker (current_buffer->mark, tem, Fcurrent_buffer ());
-  tem = Fcdr (Fcdr (info));
-  visible = !NILP (tem);
+  /* If buffer being returned to is now deleted, avoid error --
+     otherwise could get error here while unwinding to top level and
+     crash.  In that case, Fmarker_buffer returns nil now.  */
+  if (!NILP (buffer))
+    {
+      struct buffer *buf = XBUFFER (buffer);
+      struct gcpro gcpro1;
+      GCPRO1 (info);
+      set_buffer_internal (buf);
+      Fgoto_char (XCAR (info), buffer);
+      Fset_marker (buf->mark, XCDR (info), buffer);
 
 #if 0 /* We used to make the current buffer visible in the selected window
 	 if that was true previously.  That avoids some anomalies.
 	 But it creates others, and it wasn't documented, and it is simpler
 	 and cleaner never to alter the window/buffer connections.  */
-/* #### I'm certain some code somewhere depends on this behavior. --jwz */
-
-  if (visible
-      && (current_buffer != XBUFFER (XWINDOW (selected_window)->buffer)))
-    switch_to_buffer (Fcurrent_buffer (), Qnil);
+      /* I'm certain some code somewhere depends on this behavior. --jwz */
+      /* Even if it did, it certainly doesn't matter anymore, because
+         this has been the behaviour for countless XEmacs releases
+         now.  --hniksic */
+      if (visible
+	  && (current_buffer != XBUFFER (XWINDOW (selected_window)->buffer)))
+	switch_to_buffer (Fcurrent_buffer (), Qnil);
 #endif
 
-  UNGCPRO;
+      UNGCPRO;
+    }
+
   /* Free all the junk we allocated, so that a `save-excursion' comes
      for free in terms of GC junk. */
   free_marker (XMARKER (XCAR (info)));
-  free_marker (XMARKER (XCAR (XCDR (info))));
-  free_cons (XCONS (XCDR (info)));
+  free_marker (XMARKER (XCDR (info)));
   free_cons (XCONS (info));
   return Qnil;
 }
@@ -497,9 +407,10 @@ Lisp_Object
 save_current_buffer_restore (Lisp_Object buffer)
 {
   struct buffer *buf = XBUFFER (buffer);
-  if (!BUFFER_LIVE_P (buf))
-    return Qnil;
-  set_buffer_internal (buf);
+  /* Avoid signaling an error if the buffer is no longer alive.  This
+     is for consistency with save-excursion.  */
+  if (BUFFER_LIVE_P (buf))
+    set_buffer_internal (buf);
   return Qnil;
 }
 
@@ -517,7 +428,7 @@ Executes BODY just like `progn'.
   return unbind_to (speccount, Fprogn (args));
 }
 
-DEFUN ("buffer-size", Fbufsize, 0, 1, 0, /*
+DEFUN ("buffer-size", Fbuffer_size, 0, 1, 0, /*
 Return the number of characters in BUFFER.
 If BUFFER is nil, the current buffer is assumed.
 */
@@ -602,7 +513,7 @@ If BUFFER is nil, the current buffer is assumed.
 }
 
 DEFUN ("bobp", Fbobp, 0, 1, 0, /*
-Return T if point is at the beginning of the buffer.
+Return t if point is at the beginning of the buffer.
 If the buffer is narrowed, this means the beginning of the narrowed part.
 If BUFFER is nil, the current buffer is assumed.
 */
@@ -613,7 +524,7 @@ If BUFFER is nil, the current buffer is assumed.
 }
 
 DEFUN ("eobp", Feobp, 0, 1, 0, /*
-Return T if point is at the end of the buffer.
+Return t if point is at the end of the buffer.
 If the buffer is narrowed, this means the end of the narrowed part.
 If BUFFER is nil, the current buffer is assumed.
 */
@@ -626,14 +537,12 @@ If BUFFER is nil, the current buffer is assumed.
 int
 beginning_of_line_p (struct buffer *b, Bufpos pt)
 {
-  if (pt <= BUF_BEGV (b))
-    return 1;
-  return BUF_FETCH_CHAR (b, pt - 1) == '\n';
+  return pt <= BUF_BEGV (b) || BUF_FETCH_CHAR (b, pt - 1) == '\n';
 }
 
 
 DEFUN ("bolp", Fbolp, 0, 1, 0, /*
-Return T if point is at the beginning of a line.
+Return t if point is at the beginning of a line.
 If BUFFER is nil, the current buffer is assumed.
 */
        (buffer))
@@ -644,16 +553,15 @@ If BUFFER is nil, the current buffer is assumed.
 }
 
 DEFUN ("eolp", Feolp, 0, 1, 0, /*
-Return T if point is at the end of a line.
+Return t if point is at the end of a line.
 `End of a line' includes point being at the end of the buffer.
 If BUFFER is nil, the current buffer is assumed.
 */
        (buffer))
 {
   struct buffer *b = decode_buffer (buffer, 1);
-  if (BUF_PT (b) == BUF_ZV (b) || BUF_FETCH_CHAR (b, BUF_PT (b)) == '\n')
-    return Qt;
-  return Qnil;
+  return (BUF_PT (b) == BUF_ZV (b) || BUF_FETCH_CHAR (b, BUF_PT (b)) == '\n')
+    ? Qt : Qnil;
 }
 
 DEFUN ("char-after", Fchar_after, 0, 2, 0, /*
@@ -695,6 +603,30 @@ if POS is nil, the value of point is assumed.
 }
 
 
+DEFUN ("temp-directory", Ftemp_directory, 0, 0, 0, /*
+Return the pathname to the directory to use for temporary files.
+On NT/MSDOS, this is obtained from the TEMP or TMP environment variables,
+defaulting to / if they are both undefined.
+On Unix it is obtained from TMPDIR, with /tmp as the default
+*/
+       ())
+{
+  char *tmpdir;
+#if defined(WINDOWSNT) || defined(MSDOS)
+  tmpdir = getenv ("TEMP");
+  if (!tmpdir)
+    tmpdir = getenv ("TMP");
+  if (!tmpdir)
+    tmpdir = "/";
+#else /* WINDOWSNT || MSDOS */
+ tmpdir = getenv ("TMPDIR");
+ if (!tmpdir)
+   tmpdir = "/tmp";
+#endif
+
+  return build_ext_string (tmpdir, FORMAT_FILENAME);
+}
+
 DEFUN ("user-login-name", Fuser_login_name, 0, 1, 0, /*
 Return the name under which the user logged in, as a string.
 This is based on the effective uid, not the real uid.
@@ -705,33 +637,74 @@ ignored and this function returns the login name for that UID, or nil.
 */
        (uid))
 {
-  struct passwd *pw = NULL;
+  char *returned_name;
+  int local_uid;
 
   if (!NILP (uid))
     {
       CHECK_INT (uid);
-      pw = (struct passwd *) getpwuid (XINT (uid));
+      local_uid = XINT(uid);
+      returned_name = user_login_name(&local_uid);
+    }
+  else
+    {
+      returned_name = user_login_name(NULL);
+    }
+  /* #### - I believe this should return nil instead of "unknown" when pw==0
+     pw=0 is indicated by a null return from user_login_name
+  */
+  return returned_name ? build_string (returned_name) : Qnil;
+}
+
+/* This function may be called from other C routines when a
+   character string representation of the user_login_name is
+   needed but a Lisp Object is not.  The UID is passed by
+   reference.  If UID == NULL, then the USER name
+   for the user running XEmacs will be returned.  This
+   corresponds to a nil argument to Fuser_login_name.
+*/
+char*
+user_login_name (int *uid)
+{
+  struct passwd *pw = NULL;
+  
+  /* uid == NULL to return name of this user */
+  if (uid != NULL)
+    {
+      pw = getpwuid (*uid);
+      return pw ? pw->pw_name : NULL;
     }
   else
     {
-      char *user_name;
       /* #### - when euid != uid, then LOGNAME and USER are leftovers from the
 	 old environment (I site observed behavior on sunos and linux), so the
 	 environment variables should be disregarded in that case.  --Stig */
-      user_name = getenv ("LOGNAME");
+      char *user_name = getenv ("LOGNAME");
       if (!user_name)
+	user_name = getenv (
 #ifdef WINDOWSNT
-        user_name = (char *) getenv ("USERNAME"); /* it's USERNAME on NT */
-#else  /* WINDOWSNT */
-        user_name = (char *) getenv ("USER");
-#endif /* WINDOWSNT */
+			    "USERNAME" /* it's USERNAME on NT */
+#else
+			    "USER"
+#endif
+			    );
       if (user_name)
-	return build_string (user_name);
+	return (user_name);
       else
-	pw = (struct passwd *) getpwuid (geteuid ());
+	{
+	  pw = getpwuid (geteuid ());
+#ifdef __CYGWIN32__
+	  /* Since the Cygwin environment may not have an /etc/passwd,
+	     return "unknown" instead of the null if the username
+	     cannot be determined.
+	  */
+	  return pw ? pw->pw_name : "unknown";
+#else
+	  /* For all but Cygwin return NULL (nil) */
+	  return pw ? pw->pw_name : NULL;
+#endif
+	}
     }
-  /* #### - I believe this should return nil instead of "unknown" when pw==0 */
-  return pw ? build_string (pw->pw_name) : Qnil;
 }
 
 DEFUN ("user-real-login-name", Fuser_real_login_name, 0, 0, 0, /*
@@ -741,7 +714,7 @@ This ignores the environment variables LOGNAME and USER, so it differs from
 */
        ())
 {
-  struct passwd *pw = (struct passwd *) getpwuid (getuid ());
+  struct passwd *pw = getpwuid (getuid ());
   /* #### - I believe this should return nil instead of "unknown" when pw==0 */
 
 #ifdef MSDOS
@@ -775,14 +748,21 @@ DEFUN ("user-full-name", Fuser_full_name, 0, 1, 0, /*
 Return the full name of the user logged in, as a string.
 If the optional argument USER is given, then the full name for that
 user is returned, or nil.  USER may be either a login name or a uid.
+
+If USER is nil, and `user-full-name' contains a string, the
+value of `user-full-name' is returned.
 */
        (user))
 {
-  Lisp_Object user_name = (STRINGP (user) ? user : Fuser_login_name (user));
+  Lisp_Object user_name;
   struct passwd *pw = NULL;
   Lisp_Object tem;
   char *p, *q;
 
+  if (NILP (user) && STRINGP (Vuser_full_name))
+    return Vuser_full_name;
+
+  user_name = (STRINGP (user) ? user : Fuser_login_name (user));
   if (!NILP (user_name))	/* nil when nonexistent UID passed as arg */
     {
       CONST char *user_name_ext;
@@ -798,15 +778,15 @@ user is returned, or nil.  USER may be either a login name or a uid.
   /* #### - Stig sez: this should return nil instead of "unknown" when pw==0 */
   /* Ben sez: bad idea because it's likely to break something */
 #ifndef AMPERSAND_FULL_NAME
-  p = (char *) ((pw) ? USER_FULL_NAME : "unknown"); /* don't gettext */
-  q = (char *) strchr ((char *) p, ',');
+  p = ((pw) ? USER_FULL_NAME : "unknown"); /* don't gettext */
+  q = strchr (p, ',');
 #else
-  p = (char *) ((pw) ? USER_FULL_NAME : "unknown"); /* don't gettext */
-  q = (char *) strchr ((char *) p, ',');
+  p = ((pw) ? USER_FULL_NAME : "unknown"); /* don't gettext */
+  q = strchr (p, ',');
 #endif
   tem = ((!NILP (user) && !pw)
 	 ? Qnil
-	 : make_ext_string ((unsigned char *) p, (q ? q - p : strlen (p)),
+	 : make_ext_string ((Extbyte *) p, (q ? q - p : strlen (p)),
 			    FORMAT_OS));
 
 #ifdef AMPERSAND_FULL_NAME
@@ -829,12 +809,106 @@ user is returned, or nil.  USER may be either a login name or a uid.
     }
 #endif /* AMPERSAND_FULL_NAME */
 
-  p = getenv ("NAME");
-  if (p)
-    tem = build_string (p);
   return tem;
 }
 
+static char *cached_home_directory;
+
+void
+uncache_home_directory (void)
+{
+  cached_home_directory = NULL;	/* in some cases, this may cause the leaking
+				   of a few bytes */
+}
+
+char *
+get_home_directory (void)
+{
+  int output_home_warning = 0;
+
+  if (cached_home_directory == NULL)
+    {
+      if ((cached_home_directory = getenv("HOME")) == NULL)
+	{
+#if defined(WINDOWSNT) && !defined(__CYGWIN32__)
+	  char	*homedrive, *homepath;
+ 
+	  if ((homedrive = getenv("HOMEDRIVE")) != NULL &&
+	      (homepath = getenv("HOMEPATH")) != NULL)
+	    {
+	      cached_home_directory =
+		(char *) xmalloc(strlen(homedrive) + strlen(homepath) + 1);
+	      sprintf(cached_home_directory, "%s%s", homedrive, homepath);
+	    }
+	  else
+	    {
+# if 1
+	      /*
+	       * Use the current directory.
+	       * This preserves the existing XEmacs behavior, but is different
+	       * from NT Emacs.
+	       */
+	      if (initial_directory[0] != '\0')
+		{
+		  cached_home_directory = initial_directory;
+		}
+	      else
+		{
+		  /* This will probably give the wrong value */
+		  cached_home_directory = getcwd (NULL, 0);
+		}
+# else
+	      /*
+	       * This is NT Emacs behavior
+	       */
+	      cached_home_directory = "C:\\";
+	      output_home_warning = 1;
+# endif
+	    }
+#else	/* !WINDOWSNT */
+	  /*
+	   * Unix, typically.
+	   * Using "/" isn't quite right, but what should we do?
+	   * We probably should try to extract pw_dir from /etc/passwd,
+	   * before falling back to this.
+	   */
+	  cached_home_directory = "/";
+	  output_home_warning = 1;
+#endif	/* !WINDOWSNT */
+	}
+      if (initialized && output_home_warning)
+	{
+	  warn_when_safe(Quser_files_and_directories, Qwarning, "\n"
+"	XEmacs was unable to determine a good value for the user's $HOME\n"
+"	directory, and will be using the value:\n"
+"		%s\n"
+"	This is probably incorrect.",
+			 cached_home_directory
+			 );
+	}
+    }
+  return (cached_home_directory);
+}
+
+DEFUN ("user-home-directory", Fuser_home_directory, 0, 0, 0, /*
+Return the user's home directory, as a string.
+*/
+       ())
+{
+  Lisp_Object directory;
+  char *path;
+
+  directory = Qnil;
+  path = get_home_directory ();
+  if (path != NULL)
+    {
+      directory =
+	Fexpand_file_name (Fsubstitute_in_file_name (build_string (path)),
+			   Qnil);
+    }
+  return (directory);
+}
+
 DEFUN ("system-name", Fsystem_name, 0, 0, 0, /*
 Return the name of the machine you are running on, as a string.
 */
@@ -872,14 +946,11 @@ resolution finer than a second.
        ())
 {
   EMACS_TIME t;
-  Lisp_Object result[3];
 
   EMACS_GET_TIME (t);
-  XSETINT (result[0], (EMACS_SECS (t) >> 16) & 0xffff);
-  XSETINT (result[1], (EMACS_SECS (t) >> 0)  & 0xffff);
-  XSETINT (result[2], EMACS_USECS (t));
-
-  return Flist (3, result);
+  return list3 (make_int ((EMACS_SECS (t) >> 16) & 0xffff),
+		make_int ((EMACS_SECS (t) >> 0)  & 0xffff),
+		make_int (EMACS_USECS (t)));
 }
 
 DEFUN ("current-process-time", Fcurrent_process_time, 0, 0, 0, /*
@@ -914,25 +985,27 @@ time will be 0.
 }
 
 
+int lisp_to_time (Lisp_Object specified_time, time_t *result);
 int
 lisp_to_time (Lisp_Object specified_time, time_t *result)
 {
+  Lisp_Object high, low;
+
   if (NILP (specified_time))
     return time (result) != -1;
-  else
-    {
-      Lisp_Object high, low;
-      high = Fcar (specified_time);
-      CHECK_INT (high);
-      low = Fcdr (specified_time);
-      if (CONSP (low))
-	low = Fcar (low);
-      CHECK_INT (low);
-      *result = (XINT (high) << 16) + (XINT (low) & 0xffff);
-      return *result >> 16 == XINT (high);
-    }
+
+  CHECK_CONS (specified_time);
+  high = XCAR (specified_time);
+  low  = XCDR (specified_time);
+  if (CONSP (low))
+    low = XCAR (low);
+  CHECK_INT (high);
+  CHECK_INT (low);
+  *result = (XINT (high) << 16) + (XINT (low) & 0xffff);
+  return *result >> 16 == XINT (high);
 }
 
+Lisp_Object time_to_lisp (time_t the_time);
 Lisp_Object
 time_to_lisp (time_t the_time)
 {
@@ -992,7 +1065,7 @@ characters appearing in the day and month names may be incorrect.
        (format_string, _time))
 {
   time_t value;
-  int size;
+  size_t size;
 
   CHECK_STRING (format_string);
 
@@ -1067,7 +1140,7 @@ DEFUN ("encode-time", Fencode_time, 6, MANY, 0, /*
 This is the reverse operation of `decode-time', which see.
 ZONE defaults to the current time zone rule.  This can
 be a string (as from `set-time-zone-rule'), or it can be a list
-(as from `current-time-zone') or an integer (as from `decode-time')
+\(as from `current-time-zone') or an integer (as from `decode-time')
 applied without consideration for daylight savings time.
 
 You can pass more than 7 arguments; then the first six arguments
@@ -1096,7 +1169,7 @@ If you want them to stand for years in this century, you must do that yourself.
   tm.tm_isdst = -1;
 
   if (CONSP (zone))
-    zone = Fcar (zone);
+    zone = XCAR (zone);
   if (NILP (zone))
     _time = mktime (&tm);
   else
@@ -1212,16 +1285,15 @@ the data it can't find.
        (specified_time))
 {
   time_t value;
-  struct tm *t;
+  struct tm *t = NULL;
 
   if (lisp_to_time (specified_time, &value)
       && (t = gmtime (&value)) != 0)
     {
-      struct tm gmt;
+      struct tm gmt = *t;	/* Make a copy, in case localtime modifies *t.  */
       long offset;
       char *s, buf[6];
 
-      gmt = *t;		/* Make a copy, in case localtime modifies *t.  */
       t = localtime (&value);
       offset = difftm (t, &gmt);
       s = 0;
@@ -1252,7 +1324,7 @@ the data it can't find.
 /* These two values are known to load tz files in buggy implementations,
    i.e. Solaris 1 executables running under either Solaris 1 or Solaris 2.
    Their values shouldn't matter in non-buggy implementations.
-   We don't use string literals for these strings, 
+   We don't use string literals for these strings,
    since if a string in the environment is in readonly
    storage, it runs afoul of bugs in SVR4 and Solaris 2.3.
    See Sun bugs 1113095 and 1114114, ``Timezone routines
@@ -1730,7 +1802,7 @@ for the character with code N.  Returns the number of characters changed.
   get_buffer_range_char (buf, start, end, &pos, &stop, 0);
   CHECK_STRING (table);
 
-  size = string_char_length (XSTRING (table));
+  size = XSTRING_CHAR_LENGTH (table);
 
   cnt = 0;
   mc_count = begin_multiple_change (buf, pos, stop);
@@ -1790,6 +1862,7 @@ widen_buffer (struct buffer *b, int no_clip)
       /* Changing the buffer bounds invalidates any recorded current
          column.  */
       invalidate_current_column ();
+      narrow_line_number_cache (b);
     }
 }
 
@@ -1836,6 +1909,7 @@ or markers) bounding the text that should remain visible.
   MARK_CLIP_CHANGED;
   /* Changing the buffer bounds invalidates any recorded current column.  */
   invalidate_current_column ();
+  narrow_line_number_cache (buf);
   zmacs_region_stays = 0;
   return Qnil;
 }
@@ -1861,25 +1935,27 @@ save_restriction_restore (Lisp_Object data)
   Lisp_Object tem;
   int local_clip_changed = 0;
 
-  buf = XBUFFER (Fcar (data));
+  buf = XBUFFER (XCAR (data));
   if (!BUFFER_LIVE_P (buf))
-    /* someone could have killed the buffer in the meantime ... */
-    return Qnil;
-  tem = Fcdr (data);
-  newhead = XINT (Fcar (tem));
-  newtail = XINT (Fcdr (tem));
-  while (CONSP (data))
     {
-      struct Lisp_Cons *victim = XCONS (data);
-      data = victim->cdr;
-      free_cons (victim);
+      /* someone could have killed the buffer in the meantime ... */
+      free_cons (XCONS (XCDR (data)));
+      free_cons (XCONS (data));
+      return Qnil;
     }
+  tem = XCDR (data);
+  newhead = XINT (XCAR (tem));
+  newtail = XINT (XCDR (tem));
+
+  free_cons (XCONS (XCDR (data)));
+  free_cons (XCONS (data));
 
   if (newhead + newtail > BUF_Z (buf) - BUF_BEG (buf))
     {
       newhead = 0;
       newtail = 0;
     }
+
   {
     Bufpos start, end;
     Bytind bi_start, bi_end;
@@ -1894,6 +1970,7 @@ save_restriction_restore (Lisp_Object data)
       {
 	local_clip_changed = 1;
 	SET_BOTH_BUF_BEGV (buf, start, bi_start);
+	narrow_line_number_cache (buf);
       }
     if (BUF_ZV (buf) != end)
       {
@@ -2072,7 +2149,6 @@ void
 transpose_markers (Bufpos start1, Bufpos end1, Bufpos start2, Bufpos end2)
 {
   Charcount amt1, amt2, diff;
-  Bufpos mpos;
   Lisp_Object marker;
   struct buffer *buf = current_buffer;
 
@@ -2106,7 +2182,7 @@ transpose_markers (Bufpos start1, Bufpos end1, Bufpos start2, Bufpos end2)
   for (marker = BUF_MARKERS (buf); !NILP (marker);
        marker = XMARKER (marker)->chain)
     {
-      mpos = marker_position (marker);
+      Bufpos mpos = marker_position (marker);
       if (mpos >= start1 && mpos < end2)
 	{
 	  if (mpos < end1)
@@ -2179,6 +2255,7 @@ syms_of_editfns (void)
   defsymbol (&Qregion_beginning, "region-beginning");
   defsymbol (&Qregion_end, "region-end");
   defsymbol (&Qformat, "format");
+  defsymbol (&Quser_files_and_directories, "user-files-and-directories");
 
   DEFSUBR (Fchar_equal);
   DEFSUBR (Fchar_Equal);
@@ -2195,7 +2272,7 @@ syms_of_editfns (void)
   DEFSUBR (Fsave_excursion);
   DEFSUBR (Fsave_current_buffer);
 
-  DEFSUBR (Fbufsize);
+  DEFSUBR (Fbuffer_size);
   DEFSUBR (Fpoint_max);
   DEFSUBR (Fpoint_min);
   DEFSUBR (Fpoint_min_marker);
@@ -2214,11 +2291,13 @@ syms_of_editfns (void)
   DEFSUBR (Finsert_before_markers);
   DEFSUBR (Finsert_char);
 
+  DEFSUBR (Ftemp_directory);
   DEFSUBR (Fuser_login_name);
   DEFSUBR (Fuser_real_login_name);
   DEFSUBR (Fuser_uid);
   DEFSUBR (Fuser_real_uid);
   DEFSUBR (Fuser_full_name);
+  DEFSUBR (Fuser_home_directory);
   DEFSUBR (Femacs_pid);
   DEFSUBR (Fcurrent_time);
   DEFSUBR (Fcurrent_process_time);
@@ -2251,7 +2330,6 @@ vars_of_editfns (void)
 {
   staticpro (&Vsystem_name);
 #if 0
-  staticpro (&Vuser_full_name);
   staticpro (&Vuser_name);
   staticpro (&Vuser_real_name);
 #endif
@@ -2310,11 +2388,14 @@ Do not alter this.  It is for internal use only.
   zmacs_region_active_p = 0;
 
   DEFVAR_BOOL ("zmacs-region-stays", &zmacs_region_stays /*
+Whether the current command will deactivate the region.
 Commands which do not wish to affect whether the region is currently
 highlighted should set this to t.  Normally, the region is turned off after
 executing each command that did not explicitly turn it on with the function
 zmacs-activate-region. Setting this to true lets a command be non-intrusive.
 See the variable `zmacs-regions'.
+
+The same effect can be achieved using the `_' interactive specification.
 */ );
   zmacs_region_stays = 0;
 
@@ -2328,4 +2409,13 @@ is not available by any other means.
 #ifdef AMPERSAND_FULL_NAME
   Fprovide(intern("ampersand-full-name"));
 #endif
+
+  DEFVAR_LISP ("user-full-name", &Vuser_full_name /*
+*The name of the user.
+The function `user-full-name', which will return the value of this
+ variable, when called without arguments.
+This is initialized to the value of the NAME environment variable.
+*/ );
+  /* Initialized at run-time. */
+  Vuser_full_name = Qnil;
 }
