@@ -1,5 +1,5 @@
 /* The event_stream interface for X11 with Xt, and/or tty frames.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1991-5, 1997 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
    Copyright (C) 1996 Ben Wing.
 
@@ -48,15 +48,21 @@ Boston, MA 02111-1307, USA.  */
 #include <X11/CoreP.h>		/* Numerous places access the fields of
 				   a core widget directly.  We could
 				   use XtGetValues(), but ... */
+#include <X11/ShellP.h>
+
 #ifdef HAVE_XIM
 #ifdef XIM_MOTIF
 #include <Xm/Xm.h>
 #endif
 #include "lstream.h"
-#include "mule-coding.h"
+#include "file-coding.h"
 #endif
 
-#ifdef HAVE_OFFIX_DND
+#ifdef HAVE_DRAGNDROP
+#include "dragdrop.h"
+#endif
+
+#if defined (HAVE_OFFIX_DND)
 #include "offix.h"
 #endif
 
@@ -119,10 +125,6 @@ void debug_process_finalization (struct Lisp_Process *p);
 void emacs_Xt_event_handler (Widget wid, XtPointer closure, XEvent *event,
 			     Boolean *continue_to_dispatch);
 
-#ifdef EPOCH
-void dispatch_epoch_event (struct frame *f, XEvent *event, Lisp_Object type);
-#endif
-
 static int last_quit_check_signal_tick_count;
 
 Lisp_Object Qkey_mapping;
@@ -710,7 +712,7 @@ x_keysym_to_emacs_keysym (KeySym keysym, int simple_p)
 
 	   Let's hard-code in some knowledge of common keysyms introduced
 	   in recent X11 releases.  Snarfed from X11/keysymdef.h
-	   
+
 	   Probably we should add some stuff here for X11R6. */
 	switch (keysym)
 	  {
@@ -767,7 +769,7 @@ x_to_emacs_keysym (XKeyPressedEvent *event, int simple_p)
      /* simple_p means don't try too hard (ASCII only) */
 {
   KeySym keysym = 0;
-
+  
 #ifdef HAVE_XIM
   int len;
   char buffer[64];
@@ -792,7 +794,8 @@ x_to_emacs_keysym (XKeyPressedEvent *event, int simple_p)
          than passing in 0) to avoid crashes on German IRIX */
       char dummy[256];
       XLookupString (event, dummy, 200, &keysym, 0);
-      return x_keysym_to_emacs_keysym (keysym, simple_p);
+      return (IsModifierKey (keysym) || keysym == XK_Mode_switch )
+	? Qnil : x_keysym_to_emacs_keysym (keysym, simple_p);
     }
 #endif /* ! XIM_MOTIF */
 
@@ -841,7 +844,8 @@ x_to_emacs_keysym (XKeyPressedEvent *event, int simple_p)
     {
     case XLookupKeySym:
     case XLookupBoth:
-      return x_keysym_to_emacs_keysym (keysym, simple_p);
+      return (IsModifierKey (keysym) || keysym == XK_Mode_switch )
+	? Qnil : x_keysym_to_emacs_keysym (keysym, simple_p);
 
     case XLookupChars:
       {
@@ -981,20 +985,16 @@ x_event_to_emacs_event (XEvent *x_event, struct Lisp_Event *emacs_event)
 	  {
 	    Lisp_Object keysym;
 	    XKeyEvent *ev = &x_event->xkey;
-	    KeyCode keycode = ev->keycode;
-
-	    if (x_key_is_modifier_p (keycode, d)) /* it's a modifier key */
-	      return 0;
-
 	    /* This used to compute the frame from the given X window and
 	       store it here, but we really don't care about the frame. */
 	    emacs_event->channel = DEVICE_CONSOLE (d);
 	    keysym = x_to_emacs_keysym (&x_event->xkey, 0);
 
-	    /* If the emacs keysym is nil, then that means that the
-	       X keysym was NoSymbol, which probably means that
-	       we're in the midst of reading a Multi_key sequence,
-	       or a "dead" key prefix, or XIM input.  Ignore it. */
+	    /* If the emacs keysym is nil, then that means that the X
+	       keysym was either a Modifier or NoSymbol, which
+	       probably means that we're in the midst of reading a
+	       Multi_key sequence, or a "dead" key prefix, or XIM
+	       input. Ignore it. */
 	    if (NILP (keysym))
 	      return 0;
 
@@ -1113,23 +1113,23 @@ x_event_to_emacs_event (XEvent *x_event, struct Lisp_Event *emacs_event)
 	  {
 	    unsigned int state, modifiers = 0, button=0;
 	    struct frame *frame = x_any_window_to_frame (d, ev->window);
-	    unsigned char *data;
+	    Extbyte *data;
 	    unsigned long size, dtype;
 	    Lisp_Object l_type = Qnil, l_data = Qnil;
 	    Lisp_Object l_dndlist = Qnil, l_item = Qnil;
 	    struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
-	    
+
 	    GCPRO4 (l_type, l_data, l_dndlist, l_item);
 
 	    if (! frame)
 	      return 0;	/* not for us */
 	    XSETFRAME (emacs_event->channel, frame);
 
-	    emacs_event->event_type = dnd_drop_event;
+	    emacs_event->event_type = misc_user_event;
 	    emacs_event->timestamp  = DEVICE_X_LAST_SERVER_TIMESTAMP (d);
 
 	    state=DndDragButtons(x_event);
-
+	    
 	    if (state & ShiftMask)	modifiers |= MOD_SHIFT;
 	    if (state & ControlMask)	modifiers |= MOD_CONTROL;
 	    if (state & xd->MetaMask)	modifiers |= MOD_META;
@@ -1142,13 +1142,13 @@ x_event_to_emacs_event (XEvent *x_event, struct Lisp_Event *emacs_event)
 	    if (state & Button3Mask)    button = Button3;
 	    if (state & Button2Mask)    button = Button2;
 	    if (state & Button1Mask)    button = Button1;
-	    
-	    emacs_event->event.dnd_drop.modifiers = modifiers;
-	    emacs_event->event.dnd_drop.button	  = button;
 
-	    DndDropCoordinates(FRAME_X_TEXT_WIDGET(frame), x_event, 
-			       &(emacs_event->event.dnd_drop.x),
-			       &(emacs_event->event.dnd_drop.y) );
+	    emacs_event->event.misc.modifiers = modifiers;
+	    emacs_event->event.misc.button    = button;
+
+	    DndDropCoordinates(FRAME_X_TEXT_WIDGET(frame), x_event,
+			       &(emacs_event->event.misc.x),
+			       &(emacs_event->event.misc.y) );
 
 	    DndGetData(x_event,&data,&size);
 
@@ -1158,49 +1158,84 @@ x_event_to_emacs_event (XEvent *x_event, struct Lisp_Event *emacs_event)
 	      case DndFiles: /* null terminated strings, end null */
 		{
 		  int len;
+		  char *hurl = NULL;
+
 		  while (*data)
 		    {
-		      len = strlen ((char*) data);
-		      l_item = make_ext_string ((char*) data, len,
-						FORMAT_FILENAME);
-		      /* order is irrelevant */
+		      len = strlen ((char*)data);
+		      hurl = dnd_url_hexify_string ((char *)data, "file:");
+		      l_item = make_string ((Bufbyte *)hurl, strlen (hurl));
 		      l_dndlist = Fcons (l_item, l_dndlist);
-		      data += len+1;
+		      data += len + 1;
+		      xfree (hurl);
 		    }
+		  l_type = Qdragdrop_URL;
 		}
 		break;
 	      case DndText:
+		l_type = Qdragdrop_MIME;
+		l_dndlist = list1 ( list3 ( list1 ( make_string ((Bufbyte *)"text/plain", 10) ),
+					    make_string ((Bufbyte *)"8bit", 4),
+					    make_ext_string ((Extbyte *)data, 
+							     strlen((char *)data),
+							     FORMAT_CTEXT) ) );
+		break;
 	      case DndMIME:
-		/* is there a better way to format this ? */
-		l_dndlist = make_ext_string ((char*) data, strlen(data),
-					     FORMAT_BINARY);
+		/* we have to parse this in some way to extract
+		   content-type and params (in the tm way) and
+		   content encoding.
+		   OR: if data is string, let tm do the job
+		       if data is list[2], give the first two
+		       to tm...
+		*/
+		l_type = Qdragdrop_MIME;
+		l_dndlist = list1 ( make_ext_string ((Extbyte *)data,
+						     strlen((char *)data),
+						     FORMAT_BINARY) );
 		break;
 	      case DndFile:
 	      case DndDir:
 	      case DndLink:
 	      case DndExe:
-	      case DndURL: /* this could also break with FORMAT_FILENAME */
-		l_dndlist = make_ext_string ((char*) data, strlen(data),
-					     FORMAT_FILENAME);
+		{
+		  char *hurl = dnd_url_hexify_string (data, "file:");
+
+		  l_dndlist = list1 ( make_string ((Bufbyte *)hurl,
+						   strlen (hurl)) );
+		  l_type = Qdragdrop_URL;
+
+		  xfree (hurl);
+		}
+		break;
+	      case DndURL:
+		/* as it is a real URL it should already be escaped
+		   and escaping again will break them (cause % is unsave) */
+		l_dndlist = list1 ( make_ext_string ((Extbyte *)data, 
+						     strlen ((char *)data),
+						     FORMAT_FILENAME) );
+		l_type = Qdragdrop_URL;
 		break;
 	      default: /* Unknown, RawData and any other type */
-		l_dndlist = make_ext_string ((char*) data, size,
-					     FORMAT_BINARY);
+		l_dndlist = list1 ( list3 ( list1 ( make_string ((Bufbyte *)"application/octet-stream", 24) ),
+					    make_string ((Bufbyte *)"8bit", 4),
+					    make_ext_string ((Extbyte *)data,
+							     size,
+							     FORMAT_BINARY) ) );
+		l_type = Qdragdrop_MIME;
 		break;
 	      }
-	    
-	    l_type=make_int(dtype);
 
-	    emacs_event->event.dnd_drop.data = Fcons (l_type, Fcons (l_dndlist, Qnil));
+	    emacs_event->event.misc.function = Qdragdrop_drop_dispatch;
+	    emacs_event->event.misc.object = Fcons (l_type, l_dndlist);
 
 	    UNGCPRO;
 
 	    break;
 	  }
-#endif
+#endif /* HAVE_OFFIX_DND */
         if (ev->message_type == DEVICE_XATOM_WM_PROTOCOLS (d)
-            && ev->data.l[0] == DEVICE_XATOM_WM_TAKE_FOCUS (d)
-            && ev->data.l[1] == 0)
+            && (Atom) (ev->data.l[0]) == DEVICE_XATOM_WM_TAKE_FOCUS (d)
+            && (Atom) (ev->data.l[1]) == 0)
           {
             ev->data.l[1] = DEVICE_X_LAST_SERVER_TIMESTAMP (d);
           }
@@ -1320,7 +1355,7 @@ emacs_Xt_handle_focus_event (XEvent *event)
 static void
 change_frame_visibility (struct frame *f, int is_visible)
 {
-  Lisp_Object frame = Qnil;
+  Lisp_Object frame;
 
   XSETFRAME (frame, f);
 
@@ -1333,17 +1368,11 @@ change_frame_visibility (struct frame *f, int is_visible)
 	 the good 'ol double redisplay structure. */
       MARK_FRAME_WINDOWS_STRUCTURE_CHANGED (f);
       va_run_hook_with_args (Qmap_frame_hook, 1, frame);
-#ifdef EPOCH
-      dispatch_epoch_event (f, event, Qx_map);
-#endif
     }
   else if (FRAME_VISIBLE_P (f) && !is_visible)
     {
       FRAME_VISIBLE_P (f) = 0;
       va_run_hook_with_args (Qunmap_frame_hook, 1, frame);
-#ifdef EPOCH
-      dispatch_epoch_event (f, event, Qx_unmap);
-#endif
     }
   else if (FRAME_VISIBLE_P (f) * is_visible < 0)
     {
@@ -1351,16 +1380,13 @@ change_frame_visibility (struct frame *f, int is_visible)
       if (FRAME_REPAINT_P(f))
 	      MARK_FRAME_WINDOWS_STRUCTURE_CHANGED (f);
       va_run_hook_with_args (Qmap_frame_hook, 1, frame);
-#ifdef EPOCH
-      dispatch_epoch_event (f, event, Qx_map);
-#endif
     }
 }
 
 static void
 handle_map_event (struct frame *f, XEvent *event)
 {
-  Lisp_Object frame = Qnil;
+  Lisp_Object frame;
 
   XSETFRAME (frame, f);
   if (event->type == MapNotify)
@@ -1431,12 +1457,12 @@ static void
 handle_client_message (struct frame *f, XEvent *event)
 {
   struct device *d = XDEVICE (FRAME_DEVICE (f));
-  Lisp_Object frame = Qnil;
+  Lisp_Object frame;
 
   XSETFRAME (frame, f);
 
   if (event->xclient.message_type == DEVICE_XATOM_WM_PROTOCOLS (d) &&
-      event->xclient.data.l[0] == DEVICE_XATOM_WM_DELETE_WINDOW (d))
+      (Atom) (event->xclient.data.l[0]) == DEVICE_XATOM_WM_DELETE_WINDOW (d))
     {
       /* WM_DELETE_WINDOW is a misc-user event, but other ClientMessages,
 	 such as WM_TAKE_FOCUS, are eval events.  That's because delete-window
@@ -1451,7 +1477,7 @@ handle_client_message (struct frame *f, XEvent *event)
 			       list3 (Qdelete_frame, frame, Qt));
     }
   else if (event->xclient.message_type == DEVICE_XATOM_WM_PROTOCOLS (d) &&
-	   event->xclient.data.l[0] == DEVICE_XATOM_WM_TAKE_FOCUS (d))
+	   (Atom) event->xclient.data.l[0] == DEVICE_XATOM_WM_TAKE_FOCUS (d))
     {
       handle_focus_event_1 (f, 1);
 #if 0
@@ -1474,9 +1500,6 @@ handle_client_message (struct frame *f, XEvent *event)
 	}
 #endif
     }
-#ifdef EPOCH
-  dispatch_epoch_event (f, event, Qx_client_message);
-#endif
 }
 
 static void
@@ -1505,9 +1528,6 @@ emacs_Xt_handle_magic_event (struct Lisp_Event *emacs_event)
 
     case PropertyNotify:
       x_handle_property_notify (&event->xproperty);
-#ifdef EPOCH
-      dispatch_epoch_event (f, event, Qx_property_change);
-#endif
       break;
 
     case Expose:
@@ -1663,28 +1683,58 @@ emacs_Xt_remove_timeout (int id)
 {
   struct Xt_timeout *timeout, *t2;
 
+  timeout = NULL;
+  
   /* Find the timeout on the list of pending ones, if it's still there. */
-  if (!pending_timeouts) return;
-  if (id == pending_timeouts->id)
+  if (pending_timeouts)
     {
-      timeout = pending_timeouts;
-      pending_timeouts = pending_timeouts->next;
-    }
-  else
-    {
-      t2 = pending_timeouts;
-      while (t2->next && t2->next->id != id) t2 = t2->next;
-      if (! t2->next) return;
-      timeout = t2->next;
-      t2->next = t2->next->next;
+      if (id == pending_timeouts->id)
+	{
+	  timeout = pending_timeouts;
+	  pending_timeouts = pending_timeouts->next;
+	}
+      else
+	{
+	  t2 = pending_timeouts;
+	  while (t2->next && t2->next->id != id) t2 = t2->next;
+	  if ( t2->next)   /*found it */
+	    {
+	      timeout = t2->next;
+	      t2->next = t2->next->next;
+	    }
+	}
+      /* if it was pending, we have removed it from the list */
+      if (timeout)
+	XtRemoveTimeOut (timeout->interval_id);
     }
 
-  /* At this point, we've found the thing on the list of pending timeouts,
-     and removed it.
-   */
+  /* It could be that the Xt call back was already called but we didn't convert
+     into an Emacs event yet */
+  if (!timeout && completed_timeouts)
+    {
+      /* Code duplication! */
+      if (id == completed_timeouts->id)
+	{
+	  timeout = completed_timeouts;
+	  completed_timeouts = completed_timeouts->next;
+	}
+      else
+	{
+	  t2 = completed_timeouts;
+	  while (t2->next && t2->next->id != id) t2 = t2->next;
+	  if ( t2->next)   /*found it */
+	    {
+	      timeout = t2->next;
+	      t2->next = t2->next->next;
+	    }
+	}
+    }
 
-  XtRemoveTimeOut (timeout->interval_id);
-  Blocktype_free (the_Xt_timeout_blocktype, timeout);
+  /* If we found the thing on the lists of timeouts,
+     and removed it, deallocate
+  */
+  if (timeout)
+    Blocktype_free (the_Xt_timeout_blocktype, timeout);
 }
 
 static void
@@ -1870,6 +1920,24 @@ emacs_Xt_unselect_process (struct Lisp_Process *p)
   unselect_filedesc (infd);
 }
 
+static USID
+emacs_Xt_create_stream_pair (void* inhandle, void* outhandle,
+		Lisp_Object* instream, Lisp_Object* outstream, int flags)
+{
+  USID u = event_stream_unixoid_create_stream_pair
+		(inhandle, outhandle, instream, outstream, flags);
+  if (u != USID_ERROR)
+    u = USID_DONTHASH;
+  return u;
+}
+
+static USID
+emacs_Xt_delete_stream_pair (Lisp_Object instream, Lisp_Object outstream)
+{
+  event_stream_unixoid_delete_stream_pair (instream, outstream);
+  return USID_DONTHASH;
+}
+
 /* This is called from GC when a process object is about to be freed.
    If we've still got pointers to it in this file, we're gonna lose hard.
  */
@@ -1878,11 +1946,12 @@ debug_process_finalization (struct Lisp_Process *p)
 {
 #if 0 /* #### */
   int i;
-  int infd, outfd;
-  get_process_file_descriptors (p, &infd, &outfd);
+  Lisp_Object instr, outstr;
+
+  get_process_streams (p, &instr, &outstr);
   /* if it still has fds, then it hasn't been killed yet. */
-  assert (infd < 0);
-  assert (outfd < 0);
+  assert (NILP(instr));
+  assert (NILP(outstr));
   /* Better not still be in the "with input" table; we know it's got no fds. */
   for (i = 0; i < MAXDESC; i++)
     {
@@ -1917,7 +1986,7 @@ Xt_process_to_emacs_event (struct Lisp_Event *emacs_event)
 static void
 emacs_Xt_select_console (struct console *con)
 {
-  Lisp_Object console = Qnil;
+  Lisp_Object console;
   int infd;
 #ifdef HAVE_GPM
   int mousefd;
@@ -1950,7 +2019,7 @@ emacs_Xt_select_console (struct console *con)
 static void
 emacs_Xt_unselect_console (struct console *con)
 {
-  Lisp_Object console = Qnil;
+  Lisp_Object console;
   int infd;
 #ifdef HAVE_GPM
   int mousefd;
@@ -2023,12 +2092,13 @@ describe_event_window (Window window, Display *display)
   if (w)
     stderr_out (" %s", w->core.widget_class->core_class.class_name);
   f = x_any_window_to_frame (get_device_from_display (display), window);
-  if (f) {
-    char buf[500];
-    sprintf (buf, " \"%s\"", XSTRING_DATA (f->name));
-    write_string_to_stdio_stream (stderr, 0, (Bufbyte *) buf, 0, strlen (buf),
-                                  FORMAT_TERMINAL);
-  }
+  if (f)
+    {
+      char *buf = alloca_array (char, XSTRING_LENGTH (f->name) + 4);
+      sprintf (buf, " \"%s\"", XSTRING_DATA (f->name));
+      write_string_to_stdio_stream (stderr, 0, (Bufbyte *) buf, 0,
+				    strlen (buf), FORMAT_TERMINAL);
+    }
   stderr_out ("\n");
 }
 
@@ -2674,6 +2744,11 @@ XtConvertArgRec Const colorConvertArgs[] = {
   toVal->size = sizeof(type);            \
   return True /* Caller supplies `;' */
 
+/* JH: We use this because I think there's a possibility this
+   is called before the device is properly set up, in which case
+   I don't want to abort. */
+extern struct device *get_device_from_display_1 (Display *dpy);
+
 static
 Boolean EmacsXtCvtStringToPixel (
  Display     *dpy,
@@ -2686,8 +2761,10 @@ Boolean EmacsXtCvtStringToPixel (
   String       str = (String)fromVal->addr;
   XColor       screenColor;
   XColor       exactColor;
-  Screen      *screen;
+  Screen       *screen;
   Colormap     colormap;
+  Visual       *visual;
+  struct device *d;
   Status       status;
   String       params[1];
   Cardinal     num_params  = 1;
@@ -2722,13 +2799,24 @@ Boolean EmacsXtCvtStringToPixel (
   }
 
   /* Originally called XAllocNamedColor() here. */
-  status = XParseColor (DisplayOfScreen(screen), colormap, (char*)str,
-                        &screenColor);
-  if (status) {
-    status = allocate_nearest_color (DisplayOfScreen(screen), colormap,
-                                     &screenColor);
+  if ((d = get_device_from_display_1(dpy))) {
+    visual = DEVICE_X_VISUAL(d);
+    if (colormap != DEVICE_X_COLORMAP(d)) {
+      XtAppWarningMsg(the_app_con, "wierdColormap", "cvtStringToPixel",
+		      "XtToolkitWarning",
+		      "The colormap passed to cvtStringToPixel doesn't match the one registerd to the device.\n",
+		      NULL, 0);
+      status = XAllocNamedColor(dpy, colormap, (char*)str, &screenColor, &exactColor);
+    } else {
+      status = XParseColor (dpy, colormap, (char*)str, &screenColor);
+      if (status) {
+	status = allocate_nearest_color (dpy, colormap, visual, &screenColor);
+      }
+    }
+  } else {
+    /* We haven't set up this device totally yet, so just punt */
+    status = XAllocNamedColor(dpy, colormap, (char*)str, &screenColor, &exactColor);
   }
-
   if (status == 0) {
     params[0] = str;
     /* Server returns a specific error code but Xlib discards it.  Ugh */
@@ -2809,6 +2897,8 @@ vars_of_event_Xt (void)
   Xt_event_stream->select_process_cb 	= emacs_Xt_select_process;
   Xt_event_stream->unselect_process_cb 	= emacs_Xt_unselect_process;
   Xt_event_stream->quit_p_cb		= emacs_Xt_quit_p;
+  Xt_event_stream->create_stream_pair_cb= emacs_Xt_create_stream_pair;
+  Xt_event_stream->delete_stream_pair_cb= emacs_Xt_delete_stream_pair;
 
   DEFVAR_BOOL ("modifier-keys-are-sticky", &modifier_keys_are_sticky /*
 *Non-nil makes modifier keys sticky.
@@ -2841,6 +2931,28 @@ Information is displayed on stderr.  Currently defined values are:
   last_quit_check_signal_tick_count = 0;
 }
 
+/* This mess is a hack that patches the shell widget to treat visual inheritance
+   the same as colormap and depth inheritance */
+
+static XtInitProc orig_shell_init_proc;
+
+static void ShellVisualPatch(Widget wanted, Widget new,
+			     ArgList args, Cardinal *num_args)
+{
+  Widget p;
+  ShellWidget w = (ShellWidget) new;
+
+  /* first, call the original setup */
+  (*orig_shell_init_proc)(wanted, new, args, num_args);
+
+  /* if the visual isn't explicitly set, grab it from the nearest shell ancestor */
+  if (w->shell.visual == CopyFromParent) {
+    p = XtParent(w);
+    while (p && !XtIsShell(p)) p = XtParent(p);
+    if (p) w->shell.visual = ((ShellWidget)p)->shell.visual;
+  }
+}
+
 void
 init_event_Xt_late (void) /* called when already initialized */
 {
@@ -2881,4 +2993,8 @@ init_event_Xt_late (void) /* called when already initialized */
 			 XtCacheByDisplay, EmacsFreeXIMStyles);
 #endif /* XIM_XLIB */
 
+  /* insert the visual inheritance patch/hack described above */
+  orig_shell_init_proc = shellClassRec.core_class.initialize;
+  shellClassRec.core_class.initialize = ShellVisualPatch;
+
 }
