@@ -185,7 +185,7 @@ column_at_point (struct buffer *buf, Bufpos init_pos, int cur_col)
     {
       last_known_column_buffer = buf;
       last_known_column = col;
-      last_known_column_point = BUF_PT (buf);
+      last_known_column_point = init_pos;
       last_known_column_modified = BUF_MODIFF (buf);
     }
 
@@ -529,16 +529,39 @@ visible section of the buffer, and pass LINE and COL as TOPOS.
 
 #endif /* 0 */
 
+/* Helper for vmotion_1 - compute vertical pixel motion between
+   START and END in the line start cache CACHE.  This just sums
+   the line heights, including both the starting and ending lines.
+*/
+static int
+vpix_motion (line_start_cache_dynarr *cache, int start, int end)
+{
+  int i, vpix;
+
+  assert (start <= end);
+  assert (start >= 0);
+  assert (end < Dynarr_length (cache));
+  
+  vpix = 0;
+  for (i = start; i <= end; i++)
+    vpix += Dynarr_atp (cache, i)->height;
+
+  return vpix;
+}
+
 /*****************************************************************************
- vmotion
+ vmotion_1
 
  Given a starting position ORIG, move point VTARGET lines in WINDOW.
  Returns the new value for point.  If the arg ret_vpos is not nil, it is
  taken to be a pointer to an int and the number of lines actually moved is
- returned in it.
+ returned in it.  If the arg ret_vpix is not nil, it is taken to be a
+ pointer to an int and the vertical pixel height of the motion which
+ took place is returned in it.
  ****************************************************************************/
-Bufpos
-vmotion (struct window *w, Bufpos orig, int vtarget, int *ret_vpos)
+static Bufpos
+vmotion_1 (struct window *w, Bufpos orig, int vtarget,
+           int *ret_vpos, int *ret_vpix)
 {
   struct buffer *b = XBUFFER (w->buffer);
   int elt;
@@ -576,6 +599,8 @@ vmotion (struct window *w, Bufpos orig, int vtarget, int *ret_vpos)
 	}
 
       if (ret_vpos) *ret_vpos = cur_line;
+      if (ret_vpix)
+        *ret_vpix = vpix_motion (w->line_start_cache, elt, cur_line + elt);
       return ret_pt;
     }
   else if (vtarget < 0)
@@ -583,12 +608,16 @@ vmotion (struct window *w, Bufpos orig, int vtarget, int *ret_vpos)
       if (elt < -vtarget)
 	{
 	  if (ret_vpos) *ret_vpos = -elt;
+          if (ret_vpix)
+            *ret_vpix = vpix_motion (w->line_start_cache, 0, elt);
 	  /* #### This should be BUF_BEGV (b), right? */
 	  return Dynarr_atp (w->line_start_cache, 0)->start;
 	}
       else
 	{
 	  if (ret_vpos) *ret_vpos = vtarget;
+          if (ret_vpix)
+            *ret_vpix = vpix_motion (w->line_start_cache, elt + vtarget, elt);
 	  return Dynarr_atp (w->line_start_cache, elt + vtarget)->start;
 	}
     }
@@ -597,6 +626,8 @@ vmotion (struct window *w, Bufpos orig, int vtarget, int *ret_vpos)
       /* No vertical motion requested so we just return the position
          of the beginning of the current line. */
       if (ret_vpos) *ret_vpos = 0;
+      if (ret_vpix)
+        *ret_vpix = vpix_motion (w->line_start_cache, elt, elt);
 
       return Dynarr_atp (w->line_start_cache, elt)->start;
     }
@@ -604,43 +635,235 @@ vmotion (struct window *w, Bufpos orig, int vtarget, int *ret_vpos)
   RETURN_NOT_REACHED(0)	/* shut up compiler */
 }
 
-DEFUN ("vertical-motion", Fvertical_motion, 1, 2, 0, /*
+/*****************************************************************************
+ vmotion
+
+ Given a starting position ORIG, move point VTARGET lines in WINDOW.
+ Returns the new value for point.  If the arg ret_vpos is not nil, it is
+ taken to be a pointer to an int and the number of lines actually moved is
+ returned in it.
+ ****************************************************************************/
+Bufpos
+vmotion (struct window *w, Bufpos orig, int vtarget, int *ret_vpos)
+{
+  return vmotion_1 (w, orig, vtarget, ret_vpos, NULL);
+}
+
+/* Helper for Fvertical_motion.
+ */
+static
+Lisp_Object vertical_motion_1 (Lisp_Object lines, Lisp_Object window,
+                               int pixels)
+{
+  Bufpos bufpos;
+  Bufpos orig;
+  int selected;
+  int *vpos, *vpix;
+  int value=0;
+  struct window *w;
+
+  if (NILP (window))
+    window = Fselected_window (Qnil);
+
+  CHECK_WINDOW (window);
+  CHECK_INT (lines);
+
+  selected = (EQ (window, Fselected_window (Qnil)));
+
+  w = XWINDOW (window);
+
+  orig = selected ? BUF_PT (XBUFFER (w->buffer))
+                  : marker_position (w->pointm[CURRENT_DISP]);
+
+  vpos = pixels ? NULL   : &value;
+  vpix = pixels ? &value : NULL;
+
+  bufpos = vmotion_1 (w, orig, XINT (lines), vpos, vpix);
+
+  /* Note that the buffer's point is set, not the window's point. */
+  if (selected) 
+    BUF_SET_PT (XBUFFER (w->buffer), bufpos);
+  else
+    set_marker_restricted (w->pointm[CURRENT_DISP],
+			   make_int(bufpos),
+			   w->buffer);
+
+  return make_int (value);
+}
+
+DEFUN ("vertical-motion", Fvertical_motion, 1, 3, 0, /*
 Move to start of frame line LINES lines down.
 If LINES is negative, this is moving up.
-
-The optional second argument WINDOW specifies the window to use for
-parameters such as width, horizontal scrolling, and so on.
+Optional second argument is WINDOW to move in,
 the default is the selected window.
+
+Sets point to position found; this may be start of line
+or just the start of a continuation line.
+If optional third argument PIXELS is nil, returns number
+of lines moved; may be closer to zero than LINES if beginning
+or end of buffer was reached.  If PIXELS is non-nil, the
+vertical pixel height of the motion which took place is
+returned instead of the actual number of lines moved.  A
+motion of zero lines returns the height of the current line.
+
 Note that `vertical-motion' sets WINDOW's buffer's point, not
 WINDOW's point. (This differs from FSF Emacs, which buggily always
 sets current buffer's point, regardless of WINDOW.)
-
-Sets point to position found; this may be start of line
- or just the start of a continuation line.
-Returns number of lines moved; may be closer to zero than LINES
- if beginning or end of buffer was reached.
-Optional second argument is WINDOW to move in.
 */
-       (lines, window))
+       (lines, window, pixels))
 {
+  return vertical_motion_1 (lines, window, !NILP (pixels));
+}
+
+/*
+ * Like vmotion() but requested and returned movement is in pixels.
+ * HOW specifies the stopping condition.  Positive means move at least
+ * PIXELS.  Negative means at most.  Zero means as close as possible.
+ */
+Bufpos
+vmotion_pixels (Lisp_Object window, Bufpos start, int pixels, int how,
+                int *motion)
+{
+  struct window *w;
+  Bufpos eobuf, bobuf;
+  int defheight;
+  int needed;
+  int line, next;
+  int remain, abspix, dirn;
+  int elt, nelt;
+  int i;
+  line_start_cache_dynarr *cache;
+  int previous = -1;
+  int lines;
+
   if (NILP (window))
     window = Fselected_window (Qnil);
+
   CHECK_WINDOW (window);
-  {
-    Bufpos bufpos;
-    int vpos;
-    struct window *w  = XWINDOW (window);
+  w = XWINDOW (window);
 
-    CHECK_INT (lines);
+  eobuf = BUF_ZV (XBUFFER (w->buffer));
+  bobuf = BUF_BEGV (XBUFFER (w->buffer));
 
-    bufpos = vmotion (XWINDOW (window), BUF_PT (XBUFFER (w->buffer)),
-		      XINT (lines), &vpos);
+  default_face_height_and_width (window, &defheight, NULL);
 
-    /* Note that the buffer's point is set, not the window's point. */
+  /* guess num lines needed in line start cache + a few extra */
+  abspix = abs (pixels);
+  needed = (abspix + defheight-1)/defheight + 3;
+
+  dirn = (pixels >= 0) ? 1 : -1;
+
+  while (1)
+    {
+      elt = point_in_line_start_cache (w, start, needed);
+      assert (elt >= 0); /* in the cache */
+
+      cache = w->line_start_cache;
+      nelt  = Dynarr_length (cache);
+
+      *motion = 0;
+
+      if (pixels == 0)
+        /* No vertical motion requested so we just return the position
+           of the beginning of the current display line. */
+        return Dynarr_atp (cache, elt)->start;
+
+      if ((dirn < 0 && elt == 0      &&
+           Dynarr_atp (cache, elt)->start <= bobuf) ||
+          (dirn > 0 && elt == nelt-1 &&
+           Dynarr_atp (cache, elt)->end   >= eobuf))
+        return Dynarr_atp (cache, elt)->start;
+
+      remain = abspix;
+      for (i = elt; (dirn > 0) ? (i < nelt) : (i > 0); i += dirn)
+        {
+          /* cache line we're considering moving over */
+          int ii = (dirn > 0) ? i : i-1;
+
+          if (remain < 0)
+            return Dynarr_atp (cache, i)->start;
+
+          line = Dynarr_atp (cache, ii)->height;
+          next = remain - line;
+
+          /* is stopping condition satisfied? */
+          if ((how >  0  &&  remain <= 0)  ||       /* at least */
+              (how <  0  &&  next < 0)     ||       /* at most */
+              (how == 0  &&  remain <= abs (next))) /* closest */
+            return Dynarr_atp (cache, i)->start;
+
+          /* moving down and nowhere left to go? */
+          if (dirn > 0 && Dynarr_atp (cache, ii)->end >= eobuf)
+            return Dynarr_atp (cache, ii)->start;
+
+          /* take the step */
+          remain   = next;
+          *motion += dirn * line;
+
+          /* moving up and nowhere left to go? */
+          if (dirn < 0 && Dynarr_atp (cache, ii)->start <= bobuf)
+            return Dynarr_atp (cache, ii)->start;
+        }
+
+      /* get here => need more cache lines.  try again. */
+      assert (abs (*motion) > previous); /* progress? */
+      previous = abs (*motion);
+
+      lines   = (pixels < 0) ? elt : (nelt - elt);
+      needed += (remain*lines + abspix-1)/abspix + 3;
+    }
+
+  RETURN_NOT_REACHED(0)	/* shut up compiler */
+}
+
+DEFUN ("vertical-motion-pixels", Fvertical_motion_pixels, 1, 3, 0, /*
+Move to start of frame line PIXELS vertical pixels down.
+If PIXELS is negative, this is moving up.
+The actual vertical motion in pixels is returned.
+
+Optional second argument is WINDOW to move in,
+the default is the selected window.
+
+Optional third argument HOW specifies when to stop.  A value
+less than zero indicates that the motion should be no more
+than PIXELS.  A value greater than zero indicates that the
+motion should be at least PIXELS.  Any other value indicates
+that the motion should be as close as possible to PIXELS.
+*/
+       (pixels, window, how))
+{
+  Bufpos bufpos;
+  Bufpos orig;
+  int selected;
+  int motion;
+  int howto;
+  struct window *w;
+
+  if (NILP (window))
+    window = Fselected_window (Qnil);
+
+  CHECK_WINDOW (window);
+  CHECK_INT (pixels);
+
+  selected = (EQ (window, Fselected_window (Qnil)));
+
+  w = XWINDOW (window);
+
+  orig = selected ? BUF_PT (XBUFFER (w->buffer))
+                  : marker_position (w->pointm[CURRENT_DISP]);
+
+  howto = INTP (how) ? XINT (how) : 0;
+
+  bufpos = vmotion_pixels (window, orig, XINT (pixels), howto, &motion);
+
+  if (selected) 
     BUF_SET_PT (XBUFFER (w->buffer), bufpos);
+  else
+    set_marker_restricted (w->pointm[CURRENT_DISP],
+			   make_int(bufpos),
+			   w->buffer);
 
-    return make_int (vpos);
-  }
+  return make_int (motion);
 }
 
 
@@ -655,6 +878,7 @@ syms_of_indent (void)
   DEFSUBR (Fcompute_motion);
 #endif
   DEFSUBR (Fvertical_motion);
+  DEFSUBR (Fvertical_motion_pixels);
 }
 
 void
