@@ -28,8 +28,8 @@ Boston, MA 02111-1307, USA.  */
    for further details */
 
 #include <config.h>
-#include <X11/Xlocale.h>        /* More portable than <locale.h> ? */
 #include "lisp.h"
+#include <X11/Xlocale.h>        /* More portable than <locale.h> ? */
 #include "frame.h"
 #include "device.h"
 #include "window.h"
@@ -79,7 +79,12 @@ Initialize_Locale (void)
 {
   char *locale;
 
-  XtSetLanguageProc (NULL, (XtLanguageProc) NULL, NULL);
+  /* dverna - Nov. 98: ### DON'T DO THIS !!! The default XtLanguageProc
+     routine calls setlocale(LC_ALL, lang) which fucks up our lower-level
+     locale management, and especially the value of LC_NUMERIC. Anyway, since
+     at this point, we don't know yet whether we're gonna need an X11 frame,
+     we should really do it manually and not use Xlib's dumb default routine */
+  /*XtSetLanguageProc (NULL, (XtLanguageProc) NULL, NULL);*/
   if ((locale = setlocale (LC_ALL, "")) == NULL)
     {
       stderr_out ("Can't set locale.\n");
@@ -111,6 +116,8 @@ Initialize_Locale (void)
         }
     }
 
+  setlocale(LC_NUMERIC, "C");
+
   if (XSetLocaleModifiers ("") == NULL)
     {
       stderr_out ("XSetLocaleModifiers(\"\") failed\n");
@@ -243,8 +250,8 @@ XIM_init_frame (struct frame *f)
 
   if (style & XIMPreeditPosition)
     { /* Init spot to invalid values */
-      XPoint *spot = &(FRAME_X_XIC_SPOT (f));
-      spot->x = spot->y = -1;
+      XPoint *frame_spot = &(FRAME_X_XIC_SPOT (f));
+      frame_spot->x = frame_spot->y = -1;
     }
 
   XIM_SetGeometry (f);
@@ -473,8 +480,8 @@ EmacsXtCvtStringToXIMStyles (
   };
 #undef STYLE_INFO
 
-  CONST char *s   = (char *) fromVal->addr;
-  CONST char *end = s + fromVal->size;
+  char *s   = (char *) fromVal->addr;
+  char *end = s + fromVal->size;
   XIMStyles * CONST p = (XIMStyles *) toVal->addr;
   CONST char * CONST delimiter = " \t\n\r:;," ;
   CONST int  max_styles = XtNumber(emacs_XIMStyleInfo);
@@ -536,7 +543,9 @@ EmacsXtCvtStringToXIMStyles (
 
   if (p->count_styles == 0)
     {   /* No valid styles? */
-      char buf[1024];
+      char *buf = (char *)alloca (strlen (fromVal->addr)
+				  + strlen (DefaultXIMStyles)
+				  + 100);
       XrmValue new_from;
       XtAppContext the_app_con = XtDisplayToApplicationContext (dpy);
 
@@ -701,7 +710,7 @@ describe_Status (Status status)
 void
 describe_Window (Window win)
 {
-  char xwincmd[64];
+  char xwincmd[128];
   sprintf (xwincmd, "xwininfo -id 0x%x >&2; xwininfo -events -id 0x%x >&2",
            (int) win, (int) win);
   system (xwincmd);
@@ -766,19 +775,19 @@ describe_XIC (XIC xic)
 
   stderr_out ("\nInput method context attributes:\n");
   stderr_out ("Style: "); describe_XIMStyle (style);
-  stderr_out ("Client window: %x\n", client_win);
-  stderr_out ("Focus window: %x\n",  focus_win);
+  stderr_out ("Client window: %lx\n", (unsigned long int)client_win);
+  stderr_out ("Focus window: %lx\n",  (unsigned long int)focus_win);
   stderr_out ("Preedit:\n");
   describe_XRectangle ("  Area", p_area);
   describe_XRectangle ("  Area needed", p_needed);
-  stderr_out ("  foreground: %x\n", p_fg);
-  stderr_out ("  background: %x\n", p_bg);
+  stderr_out ("  foreground: %lx\n", (unsigned long int)p_fg);
+  stderr_out ("  background: %lx\n", (unsigned long int)p_bg);
   stderr_out ("  fontset: "); describe_XFontSet (p_fontset);
   stderr_out ("Status:\n");
   describe_XRectangle ("  Area", s_area);
   describe_XRectangle ("  Area needed", s_needed);
-  stderr_out ("  foreground: %x\n", s_fg);
-  stderr_out ("  background: %x\n", s_bg);
+  stderr_out ("  foreground: %lx\n", (unsigned long int)s_fg);
+  stderr_out ("  background: %lx\n", (unsigned long int)s_bg);
   stderr_out ("  fontset: \n"); describe_XFontSet (s_fontset);
   stderr_out ("XNResourceName: %s\n",  resourceName  ? resourceName  : "NULL");
   stderr_out ("XNResourceClass: %s\n", resourceClass ? resourceClass : "NULL");
