@@ -187,10 +187,8 @@ static void
 update_one_scrollbar_bs (struct frame *f, Widget sb_widget)
 {
   Boolean use_backing_store;
-  Arg al [1];
 
-  XtSetArg (al [0], XtNuseBackingStore, &use_backing_store);
-  XtGetValues (FRAME_X_TEXT_WIDGET (f), al, 1);
+  Xt_GET_VALUE (FRAME_X_TEXT_WIDGET (f), XtNuseBackingStore, &use_backing_store);
 
   if (use_backing_store && sb_widget)
     {
@@ -296,93 +294,6 @@ x_update_scrollbar_instance_status (struct window *w, int active, int size,
     }
 }
 
-/* A device method. */
-static void
-x_scrollbar_width_changed_in_frame (Lisp_Object specifier, struct frame *f,
-				    Lisp_Object oldval)
-{
-  XtWidgetGeometry req, repl;
-  Lisp_Object newval = f->scrollbar_width;
-
-  in_specifier_change_function++;
-
-  /* We want the text area to stay the same size.  So, we query the
-     current size and then adjust it for the change in the scrollbar
-     width. */
-
-  /* mirror the value in the frame resources, unless it was already
-     done. */
-  if (!in_resource_setting)
-    {
-      Arg al [1];
-      XtSetArg (al [0], XtNscrollBarWidth, XINT (newval));
-      XtSetValues (FRAME_X_TEXT_WIDGET (f), al, 1);
-    }
-
-  if (XtIsRealized (FRAME_X_CONTAINER_WIDGET (f)))
-    {
-      req.request_mode = 0;
-
-      /* the query-geometry method looks at the current value of
-	 f->scrollbar_width, so temporarily set it back to the old
-	 one. */
-      f->scrollbar_width = oldval;
-      XtQueryGeometry (FRAME_X_CONTAINER_WIDGET (f), &req, &repl);
-      f->scrollbar_width = newval;
-
-      repl.width += XINT (newval) - XINT (oldval);
-      EmacsManagerChangeSize (FRAME_X_CONTAINER_WIDGET (f), repl.width,
-			      repl.height);
-    }
-
-  in_specifier_change_function--;
-}
-
-/* A device method. */
-static void
-x_scrollbar_height_changed_in_frame (Lisp_Object specifier, struct frame *f,
-				     Lisp_Object oldval)
-{
-  XtWidgetGeometry req, repl;
-  Lisp_Object newval = f->scrollbar_height;
-
-  in_specifier_change_function++;
-
-  /* We want the text area to stay the same size.  So, we query the
-     current size and then adjust it for the change in the scrollbar
-     height. */
-
-    /* mirror the value in the frame resources, unless it was already
-       done.  Also don't do it if this is the when the frame is being
-       created -- the widgets don't even exist yet, and even if they
-       did, we wouldn't want to overwrite the resource information
-       (which might specify a user preference). */
-  if (!in_resource_setting)
-    {
-      Arg al [1];
-      XtSetArg (al [0], XtNscrollBarHeight, XINT (newval));
-      XtSetValues (FRAME_X_TEXT_WIDGET (f), al, 1);
-    }
-
-  if (XtIsRealized (FRAME_X_CONTAINER_WIDGET (f)))
-    {
-      req.request_mode = 0;
-
-      /* the query-geometry method looks at the current value of
-	 f->scrollbar_height, so temporarily set it back to the old
-	 one. */
-      f->scrollbar_height = oldval;
-      XtQueryGeometry (FRAME_X_CONTAINER_WIDGET (f), &req, &repl);
-      f->scrollbar_height = newval;
-
-      repl.height += XINT (newval) - XINT (oldval);
-      EmacsManagerChangeSize (FRAME_X_CONTAINER_WIDGET (f), repl.width,
-			      repl.height);
-    }
-
-  in_specifier_change_function--;
-}
-
 enum x_scrollbar_loop
 {
   X_FIND_SCROLLBAR_WINDOW_MIRROR,
@@ -401,25 +312,14 @@ x_scrollbar_loop (enum x_scrollbar_loop type, Lisp_Object window,
   while (mir)
     {
       struct scrollbar_instance *vinstance = mir->scrollbar_vertical_instance;
-      struct scrollbar_instance *hinstance =
-	mir->scrollbar_horizontal_instance;
-      struct frame *f;
-
-      assert (!NILP (window));
-      f = XFRAME (XWINDOW (window)->frame);
+      struct scrollbar_instance *hinstance = mir->scrollbar_horizontal_instance;
+      struct window *w = XWINDOW (window);
 
       if (mir->vchild)
-	{
-	  retval = x_scrollbar_loop (type, XWINDOW (window)->vchild,
-				     mir->vchild, id, x_win);
-	}
+	retval = x_scrollbar_loop (type, w->vchild, mir->vchild, id, x_win);
       else if (mir->hchild)
-	{
-	  retval = x_scrollbar_loop (type, XWINDOW (window)->hchild,
-				     mir->hchild, id, x_win);
-	}
-
-      if (retval != NULL)
+	retval = x_scrollbar_loop (type, w->hchild, mir->hchild, id, x_win);
+      if (retval)
 	return retval;
 
       if (hinstance || vinstance)
@@ -427,58 +327,42 @@ x_scrollbar_loop (enum x_scrollbar_loop type, Lisp_Object window,
 	  switch (type)
 	    {
 	    case X_FIND_SCROLLBAR_WINDOW_MIRROR:
-	      if ((vinstance && SCROLLBAR_X_ID (vinstance) == id)
-		  || (hinstance && SCROLLBAR_X_ID (hinstance) == id))
-		{
-		  return mir;
-		}
+	      if ((vinstance && SCROLLBAR_X_ID (vinstance) == id) ||
+		  (hinstance && SCROLLBAR_X_ID (hinstance) == id))
+		return mir;
 	      break;
 	    case X_UPDATE_FRAME_SCROLLBARS:
 	      if (!mir->vchild && !mir->hchild)
-		update_window_scrollbars (XWINDOW (window), mir, 1, 0);
+		update_window_scrollbars (w, mir, 1, 0);
 	      break;
 	    case X_SET_SCROLLBAR_POINTER:
 	      if (!mir->vchild && !mir->hchild)
 		{
-		  int loop;
+                  Widget widget;
 
-		  for (loop = 0; loop < 2; loop++)
-		    {
-		      Widget widget;
+		  widget = SCROLLBAR_X_WIDGET (hinstance);
+		  if (widget && XtIsManaged (widget))
+		    update_one_widget_scrollbar_pointer (w, widget);
 
-		      if (loop)
-			widget = SCROLLBAR_X_WIDGET (vinstance);
-		      else
-			widget = SCROLLBAR_X_WIDGET (hinstance);
-
-		      if (widget && XtIsManaged (widget))
-			{
-			  update_one_widget_scrollbar_pointer
-			    (XWINDOW (window), widget);
-			}
-		    }
+		  widget = SCROLLBAR_X_WIDGET (vinstance);
+		  if (widget && XtIsManaged (widget))
+		    update_one_widget_scrollbar_pointer (w, widget);
 		}
 	      break;
 	    case X_WINDOW_IS_SCROLLBAR:
 	      if (!mir->vchild && !mir->hchild)
 		{
-		  int loop;
+		  Widget widget;
 
-		  for (loop = 0; loop < 2; loop++)
-		    {
-		      Widget widget;
+		  widget = SCROLLBAR_X_WIDGET (hinstance);
+		  if (widget && XtIsManaged (widget) &&
+		      XtWindow (widget) == x_win)
+		    return (struct window_mirror *) 1;
 
-		      if (loop)
-			widget = SCROLLBAR_X_WIDGET (vinstance);
-		      else
-			widget = SCROLLBAR_X_WIDGET (hinstance);
-
-		      if (widget && XtIsManaged (widget))
-			{
-			  if (XtWindow (widget) == x_win)
-			    return (struct window_mirror *) 1;
-			}
-		    }
+		  widget = SCROLLBAR_X_WIDGET (vinstance);
+		  if (widget && XtIsManaged (widget) &&
+		      XtWindow (widget) == x_win)
+		    return (struct window_mirror *) 1;
 		}
 	      break;
 	    default:
@@ -487,7 +371,7 @@ x_scrollbar_loop (enum x_scrollbar_loop type, Lisp_Object window,
 	}
 
       mir = mir->next;
-      window = XWINDOW (window)->next;
+      window = w->next;
     }
 
   return NULL;
@@ -516,7 +400,7 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
   scroll_event *data = (scroll_event *) client_data;
   struct device *d = get_device_from_display (XtDisplay (widget));
   struct frame *f = x_any_window_to_frame (d, XtWindow (widget));
-  Lisp_Object win;
+  Lisp_Object win, frame;
   struct scrollbar_instance *instance;
   struct window_mirror *mirror;
 
@@ -529,6 +413,7 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
   if (NILP (win))
     return;
   instance = mirror->scrollbar_vertical_instance;
+  frame = WINDOW_FRAME (XWINDOW (win));
 
   /* It seems that this is necessary whenever signal_special_Xt_user_event()
      is called.  #### Why??? */
@@ -537,11 +422,11 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
   switch (data->action)
     {
     case SCROLLBAR_LINE_UP:
-      signal_special_Xt_user_event (win, Qscrollbar_line_up, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_line_up, win);
       break;
 
     case SCROLLBAR_LINE_DOWN:
-      signal_special_Xt_user_event (win, Qscrollbar_line_down, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_line_down, win);
       break;
 
       /* The Athena scrollbar paging behavior is that of xterms.
@@ -558,11 +443,11 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
 
 	if (line > -1.0)
 	  line = -1.0;
-	signal_special_Xt_user_event (win, Qscrollbar_page_up,
+	signal_special_Xt_user_event (frame, Qscrollbar_page_up,
 				      Fcons (win, make_int ((int) line)));
       }
 #else
-      signal_special_Xt_user_event (win, Qscrollbar_page_up,
+      signal_special_Xt_user_event (frame, Qscrollbar_page_up,
 				    Fcons (win, Qnil));
 #endif
       break;
@@ -581,23 +466,23 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
 	  {
 	    if (line < 1.0)
 	      line = 1.0;
-	    signal_special_Xt_user_event (win, Qscrollbar_page_down,
+	    signal_special_Xt_user_event (frame, Qscrollbar_page_down,
 					  Fcons (win,
 						 make_int ((int) line)));
 	  }
       }
 #else
-      signal_special_Xt_user_event (win, Qscrollbar_page_down,
+      signal_special_Xt_user_event (frame, Qscrollbar_page_down,
 				    Fcons (win, Qnil));
 #endif
       break;
 
     case SCROLLBAR_TOP:
-      signal_special_Xt_user_event (win, Qscrollbar_to_top, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_to_top, win);
       break;
 
     case SCROLLBAR_BOTTOM:
-      signal_special_Xt_user_event (win, Qscrollbar_to_bottom, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_to_bottom, win);
       break;
 
 
@@ -652,14 +537,15 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
 	      }
 	    else
 	      {
-		value = (SCROLLBAR_X_POS_DATA (instance).minimum
-			 + (((double)
-			     (SCROLLBAR_X_VDRAG_ORIG_WINDOW_START (instance)
-			      - SCROLLBAR_X_POS_DATA (instance).minimum)
-			     * (data->slider_value -
-				SCROLLBAR_X_POS_DATA (instance).minimum))
-			    / (SCROLLBAR_X_VDRAG_ORIG_VALUE (instance)
-			       - SCROLLBAR_X_POS_DATA (instance).minimum)));
+		value = (int)
+		  (SCROLLBAR_X_POS_DATA (instance).minimum
+		   + (((double)
+		       (SCROLLBAR_X_VDRAG_ORIG_WINDOW_START (instance)
+			- SCROLLBAR_X_POS_DATA (instance).minimum)
+		       * (data->slider_value -
+			  SCROLLBAR_X_POS_DATA (instance).minimum))
+		      / (SCROLLBAR_X_VDRAG_ORIG_VALUE (instance)
+			 - SCROLLBAR_X_POS_DATA (instance).minimum)));
 	      }
 	  }
 	else
@@ -676,14 +562,16 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
 	      }
 	    else
 	      {
-		value = (SCROLLBAR_X_VDRAG_ORIG_WINDOW_START (instance)
-			 + (((double) (SCROLLBAR_X_POS_DATA (instance).maximum
-				       - SCROLLBAR_X_VDRAG_ORIG_WINDOW_START (instance))
-			     * (data->slider_value
-				- SCROLLBAR_X_VDRAG_ORIG_VALUE (instance)))
-			    / (SCROLLBAR_X_POS_DATA (instance).maximum
-			       - SCROLLBAR_X_POS_DATA (instance).slider_size
-			       - SCROLLBAR_X_VDRAG_ORIG_VALUE (instance))));
+		value = (int)
+		  (SCROLLBAR_X_VDRAG_ORIG_WINDOW_START (instance)
+		   + (((double)
+		       (SCROLLBAR_X_POS_DATA (instance).maximum
+			- SCROLLBAR_X_VDRAG_ORIG_WINDOW_START (instance))
+		       * (data->slider_value
+			  - SCROLLBAR_X_VDRAG_ORIG_VALUE (instance)))
+		      / (SCROLLBAR_X_POS_DATA (instance).maximum
+			 - SCROLLBAR_X_POS_DATA (instance).slider_size
+			 - SCROLLBAR_X_VDRAG_ORIG_VALUE (instance))));
 	      }
 	  }
 #else
@@ -696,7 +584,7 @@ x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
 	if (value < SCROLLBAR_X_POS_DATA (instance).minimum)
 	  value = SCROLLBAR_X_POS_DATA (instance).minimum;
 
-	signal_special_Xt_user_event (win, Qscrollbar_vertical_drag,
+	signal_special_Xt_user_event (frame, Qscrollbar_vertical_drag,
 				      Fcons (win, make_int (value)));
       }
       break;
@@ -716,7 +604,7 @@ x_update_horizontal_scrollbar_callback (Widget widget, LWLIB_ID id,
   scroll_event *data = (scroll_event *) client_data;
   struct device *d = get_device_from_display (XtDisplay (widget));
   struct frame *f = x_any_window_to_frame (d, XtWindow (widget));
-  Lisp_Object win;
+  Lisp_Object win, frame;
   struct window_mirror *mirror;
 
   if (!f)
@@ -727,6 +615,7 @@ x_update_horizontal_scrollbar_callback (Widget widget, LWLIB_ID id,
 
   if (NILP (win))
     return;
+  frame = WINDOW_FRAME (XWINDOW (win));
 
   /* It seems that this is necessary whenever signal_special_Xt_user_event()
      is called.  #### Why??? */
@@ -735,22 +624,22 @@ x_update_horizontal_scrollbar_callback (Widget widget, LWLIB_ID id,
   switch (data->action)
     {
     case SCROLLBAR_LINE_UP:
-      signal_special_Xt_user_event (win, Qscrollbar_char_left, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_char_left, win);
       break;
     case SCROLLBAR_LINE_DOWN:
-      signal_special_Xt_user_event (win, Qscrollbar_char_right, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_char_right, win);
       break;
     case SCROLLBAR_PAGE_UP:
-      signal_special_Xt_user_event (win, Qscrollbar_page_left, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_page_left, win);
       break;
     case SCROLLBAR_PAGE_DOWN:
-      signal_special_Xt_user_event (win, Qscrollbar_page_right, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_page_right, win);
       break;
     case SCROLLBAR_TOP:
-      signal_special_Xt_user_event (win, Qscrollbar_to_left, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_to_left, win);
       break;
     case SCROLLBAR_BOTTOM:
-      signal_special_Xt_user_event (win, Qscrollbar_to_right, win);
+      signal_special_Xt_user_event (frame, Qscrollbar_to_right, win);
       break;
     case SCROLLBAR_CHANGE:
       inhibit_slider_size_change = 0;
@@ -760,11 +649,11 @@ x_update_horizontal_scrollbar_callback (Widget widget, LWLIB_ID id,
       /* #### Fix the damn toolkit code so they all work the same way.
          Lucid is the one mostly wrong.*/
 #if defined (LWLIB_SCROLLBARS_LUCID) || defined (LWLIB_SCROLLBARS_ATHENA3D)
-      signal_special_Xt_user_event (win, Qscrollbar_horizontal_drag,
+      signal_special_Xt_user_event (frame, Qscrollbar_horizontal_drag,
 				    (Fcons
 				     (win, make_int (data->slider_value))));
 #else
-      signal_special_Xt_user_event (win, Qscrollbar_horizontal_drag,
+      signal_special_Xt_user_event (frame, Qscrollbar_horizontal_drag,
 				    (Fcons
 				     (win,
 				      make_int (data->slider_value - 1))));
@@ -778,7 +667,7 @@ x_update_horizontal_scrollbar_callback (Widget widget, LWLIB_ID id,
 static void
 x_scrollbar_pointer_changed_in_window (struct window *w)
 {
-  Lisp_Object window = Qnil;
+  Lisp_Object window;
 
   XSETWINDOW (window, w);
   x_scrollbar_loop (X_SET_SCROLLBAR_POINTER, window, find_window_mirror (w),
@@ -851,8 +740,6 @@ console_type_create_scrollbar_x (void)
   CONSOLE_HAS_METHOD (x, create_scrollbar_instance);
   CONSOLE_HAS_METHOD (x, update_scrollbar_instance_values);
   CONSOLE_HAS_METHOD (x, update_scrollbar_instance_status);
-  CONSOLE_HAS_METHOD (x, scrollbar_width_changed_in_frame);
-  CONSOLE_HAS_METHOD (x, scrollbar_height_changed_in_frame);
   CONSOLE_HAS_METHOD (x, scrollbar_pointer_changed_in_window);
 #ifdef MEMORY_USAGE_STATS
   CONSOLE_HAS_METHOD (x, compute_scrollbar_instance_usage);
