@@ -25,11 +25,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include <setjmp.h>
 
-#if !defined(VMS) || !defined(LINK_CRTL_SHARE) || !defined(SHAREABLE_LIB_BUG)
 extern char **environ;
-#else
-extern noshare char **environ;
-#endif /* VMS sharable environ bug */
 
 struct emacs_tty;
 int emacs_get_tty (int fd, struct emacs_tty *settings);
@@ -47,6 +43,9 @@ void set_descriptor_non_blocking (int fd);
 
 void wait_without_blocking (void);
 
+int get_pty_max_bytes (int fd);
+Bufbyte get_eof_char (int fd);
+
 /* Wait for subprocess with process id `pid' to terminate and
    make sure it will get eliminated (not remain forever as a zombie) */
 void wait_for_termination (int pid);
@@ -79,9 +78,9 @@ void reset_poll_for_quit (void);
 extern JMP_BUF break_system_call_jump;
 extern volatile int can_break_system_calls;
 
-int sys_write_1 (int fildes, CONST void *buf, unsigned int nbyte,
+int sys_write_1 (int fildes, CONST void *buf, size_t nbyte,
 		 int allow_quit);
-int sys_read_1 (int fildes, void *buf, unsigned int nbyte,
+int sys_read_1 (int fildes, void *buf, size_t nbyte,
 		int allow_quit);
 
 /* Call these functions if you want to change some terminal parameter --
@@ -154,21 +153,6 @@ int rename (CONST char *from, CONST char *to);
 int dup2 (int oldd, int newd);
 #endif
 
-#ifdef VMS
-void init_vms_input (void);
-
-int vms_truncate (char *fn);
-int vlimit (void);       /* BTW, name conflicts with unix syscall */
-int define_logical_name (char *varname, char *string)
-int delete_logical_name (char *varname);
-int rename_sans_version (char *from, char *to);
-char *vmserrstr (int status);
-char *sys_translate_vms (char *vfile);
-char *sys_translate_unix (char *ufile);
-int creat_copy_attrs (char *old, char *new);
-int link (char *file, char *new);
-#endif /* VMS */
-
 #ifndef HAVE_STRERROR
 /* X11R6 defines strerror as a macro */
 # ifdef strerror
@@ -177,8 +161,21 @@ int link (char *file, char *new);
 CONST char *strerror (int);
 #endif
 
+#ifdef WINDOWSNT
+void mswindows_set_errno (unsigned long win32_error);
+void mswindows_set_last_errno (void);
+#endif
+
+int interruptible_open (CONST char *path, int oflag, int mode);
+
 #ifndef HAVE_H_ERRNO
 extern int h_errno;
 #endif
 
+#ifdef HAVE_REALPATH
+#define xrealpath realpath
+#else
+char *xrealpath(CONST char *path, char resolved_path []);
+#endif
+
 #endif /* _XEMACS_SYSDEP_H_ */
