@@ -26,6 +26,15 @@ Boston, MA 02111-1307, USA.  */
 #include <sys/errno.h>          /* <errno.h> does not always imply this */
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifndef INCLUDED_FCNTL
+# define INCLUDED_FCNTL
+# include <fcntl.h>
+#endif /* INCLUDED_FCNTL */
+
 /* Load sys/types.h if not already loaded.
    In some systems loading it twice is suicidal.  */
 #ifndef makedev
@@ -35,42 +44,17 @@ Boston, MA 02111-1307, USA.  */
 #include <sys/stat.h>
 #include <sys/param.h>
 
-#if NeXT
-/* what is needed from here?  Do others need it too? */
+#if defined (NeXT) || defined(__CYGWIN32__)
+/* what is needed from here?  Do others need it too?
+ O_BINARY is in here under cygwin. */
 # include <sys/fcntl.h>
 #endif /* NeXT */
 
-#ifdef VMS
-#ifndef __GNUC__
-#include <file.h>
-#endif
-#include <rms.h>
-#include <rmsdef.h>
-#include <fab.h>
-#include <nam.h>
-#include <perror.h>
-#include <stddef.h>
-#endif
-
 #ifdef WINDOWSNT
 #include <io.h>
+#include <direct.h>
 #endif
 
-#if 0
-#ifdef WINDOWSNT
-#include <windows.h>
-#include <stdlib.h>	/* for proper declaration of environ */
-#include <fcntl.h>
-#include "nt.h"
-#define _P_NOWAIT 1	/* from process.h */
-#endif
-#endif
-
-#ifdef MSDOS
-#include <dos.h>
-#include "msdos.h"
-#endif /* MSDOS */
-
 #ifndef O_RDONLY
 #define O_RDONLY 0
 #endif
@@ -83,6 +67,55 @@ Boston, MA 02111-1307, USA.  */
 #define O_RDWR 2
 #endif
 
+/* file opening defaults */
+#ifndef OPEN_BINARY
+#ifdef O_BINARY
+#define OPEN_BINARY	O_BINARY
+#else
+#define OPEN_BINARY	(0)
+#endif
+#endif
+
+#ifndef OPEN_TEXT
+#ifdef O_TEXT
+#define OPEN_TEXT	O_TEXT
+#else
+#define OPEN_TEXT	(0)
+#endif
+#endif
+
+#ifndef CREAT_MODE
+#ifdef WINDOWSNT
+#define CREAT_MODE	(S_IREAD | S_IWRITE)
+#else
+#define CREAT_MODE	(0666)
+#endif
+#endif
+
+#ifndef READ_TEXT
+#ifdef O_TEXT
+#define READ_TEXT "rt"
+#else
+#define READ_TEXT "r"
+#endif
+#endif
+
+#ifndef READ_BINARY
+#ifdef O_BINARY
+#define READ_BINARY "rb"
+#else
+#define READ_BINARY "r"
+#endif
+#endif
+
+#ifndef WRITE_BINARY
+#ifdef O_BINARY
+#define WRITE_BINARY "wb"
+#else
+#define WRITE_BINARY "w"
+#endif
+#endif
+
 /* if system does not have symbolic links, it does not have lstat.
    In that case, use ordinary stat instead.  */
 
@@ -156,7 +189,7 @@ Boston, MA 02111-1307, USA.  */
 #define S_ISNWK(m) (((m) & S_IFMT) == S_IFNWK)
 #endif
 
-#if !defined (USG) && !defined (VMS) && !defined (WINDOWSNT)
+#if !defined (USG)  && !defined (WINDOWSNT)
 # define HAVE_FSYNC
 #endif
 
@@ -173,10 +206,124 @@ Boston, MA 02111-1307, USA.  */
 # define FD_CLOEXEC 1
 #endif
 
+/* Emacs needs to use its own definitions of certain system calls on
+   some systems (like SunOS 4.1 and USG systems, where the read system
+   call is interruptible but Emacs expects it not to be; and under
+   MULE, where all filenames need to be converted to external format).
+   To do this, we #define read to be sys_read, which is defined in
+   sysdep.c.  We first #undef read, in case some system file defines
+   read as a macro.  sysdep.c doesn't encapsulate read, so the call to
+   read inside of sys_read will do the right thing.
+
+   DONT_ENCAPSULATE is used in files such as sysdep.c that want to
+   call the actual system calls rather than the encapsulated versions.
+   Those files can call sys_read to get the (possibly) encapsulated
+   versions.
+
+   IMPORTANT: the redefinition of the system call must occur *after* the
+   inclusion of any header files that declare or define the system call;
+   otherwise lots of unfriendly things can happen.  This goes for all
+   encapsulated system calls.
+
+   We encapsulate the most common system calls here; we assume their
+   declarations are in one of the standard header files included above.
+   Other encapsulations are declared in the appropriate sys*.h file. */
+
+#ifdef ENCAPSULATE_READ
+int sys_read (int, void *, size_t);
+#endif
+#if defined (ENCAPSULATE_READ) && !defined (DONT_ENCAPSULATE)
+# undef read
+# define read sys_read
+#endif
+#if !defined (ENCAPSULATE_READ) && defined (DONT_ENCAPSULATE)
+# define sys_read read
+#endif
+
+#ifdef ENCAPSULATE_WRITE
+int sys_write (int, CONST void *, size_t);
+#endif
+#if defined (ENCAPSULATE_WRITE) && !defined (DONT_ENCAPSULATE)
+# undef write
+# define write sys_write
+#endif
+#if !defined (ENCAPSULATE_WRITE) && defined (DONT_ENCAPSULATE)
+# define sys_write write
+#endif
+
+#ifdef ENCAPSULATE_OPEN
+int sys_open (CONST char *, int, ...);
+#endif
+#if defined (ENCAPSULATE_OPEN) && !defined (DONT_ENCAPSULATE)
+# undef open
+# define open sys_open
+#endif
+#if !defined (ENCAPSULATE_OPEN) && defined (DONT_ENCAPSULATE)
+# define sys_open open
+#endif
+
+#ifdef ENCAPSULATE_CLOSE
+int sys_close (int);
+#endif
+#if defined (ENCAPSULATE_CLOSE) && !defined (DONT_ENCAPSULATE)
+# undef close
+# define close sys_close
+#endif
+#if !defined (ENCAPSULATE_CLOSE) && defined (DONT_ENCAPSULATE)
+# define sys_close close
+#endif
+
+/* Now the stdio versions ... */
+
+#ifdef ENCAPSULATE_FREAD
+size_t sys_fread (void *, size_t, size_t, FILE *);
+#endif
+#if defined (ENCAPSULATE_FREAD) && !defined (DONT_ENCAPSULATE)
+# undef fread
+# define fread sys_fread
+#endif
+#if !defined (ENCAPSULATE_FREAD) && defined (DONT_ENCAPSULATE)
+# define sys_fread fread
+#endif
+
+#ifdef ENCAPSULATE_FWRITE
+size_t sys_fwrite (CONST void *, size_t, size_t, FILE *);
+#endif
+#if defined (ENCAPSULATE_FWRITE) && !defined (DONT_ENCAPSULATE)
+# undef fwrite
+# define fwrite sys_fwrite
+#endif
+#if !defined (ENCAPSULATE_FWRITE) && defined (DONT_ENCAPSULATE)
+# define sys_fwrite fwrite
+#endif
+
+#ifdef ENCAPSULATE_FOPEN
+FILE *sys_fopen (CONST char *, CONST char *);
+#endif
+#if defined (ENCAPSULATE_FOPEN) && !defined (DONT_ENCAPSULATE)
+# undef fopen
+# define fopen sys_fopen
+#endif
+#if !defined (ENCAPSULATE_FOPEN) && defined (DONT_ENCAPSULATE)
+# define sys_fopen fopen
+#endif
+
+#ifdef ENCAPSULATE_FCLOSE
+int sys_fclose (FILE *);
+#endif
+#if defined (ENCAPSULATE_FCLOSE) && !defined (DONT_ENCAPSULATE)
+# undef fclose
+# define fclose sys_fclose
+#endif
+#if !defined (ENCAPSULATE_FCLOSE) && defined (DONT_ENCAPSULATE)
+# define sys_fclose fclose
+#endif
+
+
 /* encapsulations: file-information calls */
 
 #ifdef ENCAPSULATE_ACCESS
-extern int sys_access (CONST char *path, int mode);
+int sys_access (CONST char *path, int mode);
 #endif
 #if defined (ENCAPSULATE_ACCESS) && !defined (DONT_ENCAPSULATE)
 # undef access
@@ -187,7 +334,7 @@ extern int sys_access (CONST char *path, int mode);
 #endif
 
 #ifdef ENCAPSULATE_EACCESS
-extern int sys_eaccess (CONST char *path, int mode);
+int sys_eaccess (CONST char *path, int mode);
 #endif
 #if defined (ENCAPSULATE_EACCESS) && !defined (DONT_ENCAPSULATE)
 # undef eaccess
@@ -198,7 +345,7 @@ extern int sys_eaccess (CONST char *path, int mode);
 #endif
 
 #ifdef ENCAPSULATE_LSTAT
-extern int sys_lstat (CONST char *path, struct stat *buf);
+int sys_lstat (CONST char *path, struct stat *buf);
 #endif
 #if defined (ENCAPSULATE_LSTAT) && !defined (DONT_ENCAPSULATE)
 # undef lstat
@@ -209,7 +356,7 @@ extern int sys_lstat (CONST char *path, struct stat *buf);
 #endif
 
 #ifdef ENCAPSULATE_READLINK
-extern int sys_readlink (CONST char *path, char *buf, int bufsiz);
+int sys_readlink (CONST char *path, char *buf, size_t bufsiz);
 #endif
 #if defined (ENCAPSULATE_READLINK) && !defined (DONT_ENCAPSULATE)
 # undef readlink
@@ -220,7 +367,7 @@ extern int sys_readlink (CONST char *path, char *buf, int bufsiz);
 #endif
 
 #ifdef ENCAPSULATE_STAT
-extern int sys_stat (CONST char *path, struct stat *buf);
+int sys_stat (CONST char *path, struct stat *buf);
 #endif
 #if defined (ENCAPSULATE_STAT) && !defined (DONT_ENCAPSULATE)
 # undef stat
@@ -234,7 +381,7 @@ extern int sys_stat (CONST char *path, struct stat *buf);
 /* encapsulations: file-manipulation calls */
 
 #ifdef ENCAPSULATE_CHMOD
-extern int sys_chmod (CONST char *path, int mode);
+int sys_chmod (CONST char *path, mode_t mode);
 #endif
 #if defined (ENCAPSULATE_CHMOD) && !defined (DONT_ENCAPSULATE)
 # undef chmod
@@ -245,7 +392,7 @@ extern int sys_chmod (CONST char *path, int mode);
 #endif
 
 #ifdef ENCAPSULATE_CREAT
-extern int sys_creat (CONST char *path, int mode);
+int sys_creat (CONST char *path, mode_t mode);
 #endif
 #if defined (ENCAPSULATE_CREAT) && !defined (DONT_ENCAPSULATE)
 # undef creat
@@ -256,7 +403,7 @@ extern int sys_creat (CONST char *path, int mode);
 #endif
 
 #ifdef ENCAPSULATE_LINK
-extern int sys_link (CONST char *existing, CONST char *new);
+int sys_link (CONST char *existing, CONST char *new);
 #endif
 #if defined (ENCAPSULATE_LINK) && !defined (DONT_ENCAPSULATE)
 # undef link
@@ -267,7 +414,7 @@ extern int sys_link (CONST char *existing, CONST char *new);
 #endif
 
 #ifdef ENCAPSULATE_RENAME
-extern int sys_rename (CONST char *old, CONST char *new);
+int sys_rename (CONST char *old, CONST char *new);
 #endif
 #if defined (ENCAPSULATE_RENAME) && !defined (DONT_ENCAPSULATE)
 # undef rename
@@ -278,7 +425,7 @@ extern int sys_rename (CONST char *old, CONST char *new);
 #endif
 
 #ifdef ENCAPSULATE_SYMLINK
-extern int sys_symlink (CONST char *name1, CONST char *name2);
+int sys_symlink (CONST char *name1, CONST char *name2);
 #endif
 #if defined (ENCAPSULATE_SYMLINK) && !defined (DONT_ENCAPSULATE)
 # undef symlink
@@ -289,7 +436,7 @@ extern int sys_symlink (CONST char *name1, CONST char *name2);
 #endif
 
 #ifdef ENCAPSULATE_UNLINK
-extern int sys_unlink (CONST char *path);
+int sys_unlink (CONST char *path);
 #endif
 #if defined (ENCAPSULATE_UNLINK) && !defined (DONT_ENCAPSULATE)
 # undef unlink
@@ -300,7 +447,7 @@ extern int sys_unlink (CONST char *path);
 #endif
 
 #ifdef ENCAPSULATE_EXECVP
-extern int sys_execvp (CONST char *, char * CONST *);
+int sys_execvp (CONST char *, char * CONST *);
 #endif
 #if defined (ENCAPSULATE_EXECVP) && !defined (DONT_ENCAPSULATE)
 # undef execvp
