@@ -41,11 +41,13 @@ Boston, MA 02111-1307, USA.  */
 
 static void
 x_draw_blank_toolbar_button (struct frame *f, int x, int y, int width,
-			     int height, int threed)
+			     int height, int threed, int border_width,
+			     int vertical)
 {
   struct device *d = XDEVICE (f->device);
   EmacsFrame ef = (EmacsFrame) FRAME_X_TEXT_WIDGET (f);
   int shadow_thickness = ef->emacs_frame.toolbar_shadow_thickness;
+  int sx = x, sy = y, swidth = width, sheight = height;
 
   Display *dpy = DEVICE_X_DISPLAY (d);
   Window x_win = XtWindow (FRAME_X_TEXT_WIDGET (f));
@@ -64,14 +66,35 @@ x_draw_blank_toolbar_button (struct frame *f, int x, int y, int width,
       bottom_shadow_gc = background_gc;
     }
 
+  if (vertical)
+    {
+      sx += border_width;
+      swidth -= 2 * border_width;
+    }
+  else
+    {
+      sy += border_width;
+      sheight -= 2 * border_width;
+    }
+
   /* Draw the outline. */
-  x_output_shadows (f, x, y, width, height, top_shadow_gc,
+  x_output_shadows (f, sx, sy, swidth, sheight, top_shadow_gc,
 		    bottom_shadow_gc, background_gc, shadow_thickness);
 
   /* Blank the middle. */
-  XFillRectangle (dpy, x_win, background_gc, x + shadow_thickness,
-		  y + shadow_thickness, width - shadow_thickness * 2,
-		  height - shadow_thickness * 2);
+  XFillRectangle (dpy, x_win, background_gc, sx + shadow_thickness,
+		  sy + shadow_thickness, swidth - shadow_thickness * 2,
+		  sheight - shadow_thickness * 2);
+
+  /* Do the border */
+  XFillRectangle (dpy, x_win, background_gc, x, y,
+		  (vertical ? border_width : width),
+		  (vertical ? height : border_width));
+  XFillRectangle (dpy, x_win, background_gc,
+		  (vertical ? sx + swidth : x),
+		  (vertical ? y : sy + sheight),
+		  (vertical ? border_width : width),
+		  (vertical ? height : border_width));
 }
 
 static void
@@ -80,6 +103,7 @@ x_output_toolbar_button (struct frame *f, Lisp_Object button)
   struct device *d = XDEVICE (f->device);
   EmacsFrame ef = (EmacsFrame) FRAME_X_TEXT_WIDGET (f);
   int shadow_thickness = ef->emacs_frame.toolbar_shadow_thickness;
+  int x_adj, y_adj, width_adj, height_adj;
 
   Display *dpy = DEVICE_X_DISPLAY (d);
   Window x_win = XtWindow (FRAME_X_TEXT_WIDGET (f));
@@ -88,6 +112,23 @@ x_output_toolbar_button (struct frame *f, Lisp_Object button)
   struct toolbar_button *tb = XTOOLBAR_BUTTON (button);
   struct Lisp_Image_Instance *p;
   struct window *w;
+  int vertical = tb->vertical;
+  int border_width = tb->border_width;
+
+  if (vertical)
+    {
+      x_adj = border_width;
+      width_adj = - 2 * border_width;
+      y_adj = 0;
+      height_adj = 0;
+    }
+  else
+    {
+      x_adj = 0;
+      width_adj = 0;
+      y_adj = border_width;
+      height_adj = - 2 * border_width;
+    }
 
   XSETFRAME (frame, f);
   window = FRAME_LAST_NONMINIBUF_WINDOW (f);
@@ -116,13 +157,27 @@ x_output_toolbar_button (struct frame *f, Lisp_Object button)
   background_gc = FRAME_X_TOOLBAR_BLANK_BACKGROUND_GC (f);
 
   /* Draw the outline. */
-  x_output_shadows (f, tb->x, tb->y, tb->width, tb->height, top_shadow_gc,
+  x_output_shadows (f, tb->x + x_adj, tb->y + y_adj,
+		    tb->width + width_adj, tb->height + height_adj,
+		    top_shadow_gc,
 		    bottom_shadow_gc, background_gc, shadow_thickness);
 
   /* Clear the pixmap area. */
-  XFillRectangle (dpy, x_win, background_gc, tb->x + shadow_thickness,
-		  tb->y + shadow_thickness, tb->width - shadow_thickness * 2,
-		  tb->height - shadow_thickness * 2);
+  XFillRectangle (dpy, x_win, background_gc, tb->x + x_adj + shadow_thickness,
+		  tb->y + y_adj + shadow_thickness,
+		  tb->width + width_adj - shadow_thickness * 2,
+		  tb->height + height_adj - shadow_thickness * 2);
+
+  /* Do the border. */
+  XFillRectangle (dpy, x_win, background_gc, tb->x, tb->y,
+		  (vertical ? border_width : tb->width),
+		  (vertical ? tb->height : border_width));
+
+  XFillRectangle (dpy, x_win, background_gc,
+		  (vertical ? tb->x + tb->width - border_width : tb->x),
+		  (vertical ? tb->y : tb->y + tb->height - border_width),
+		  (vertical ? border_width : tb->width),
+		  (vertical ? tb->height : border_width));
 
   background_gc = FRAME_X_TOOLBAR_PIXMAP_BACKGROUND_GC (f);
 
@@ -135,10 +190,10 @@ x_output_toolbar_button (struct frame *f, Lisp_Object button)
 
   if (IMAGE_INSTANCEP (instance))
     {
-      int width = tb->width - shadow_thickness * 2;
-      int height = tb->height - shadow_thickness * 2;
-      int x_offset = shadow_thickness;
-      int y_offset = shadow_thickness;
+      int width = tb->width + width_adj - shadow_thickness * 2;
+      int height = tb->height + height_adj - shadow_thickness * 2;
+      int x_offset = x_adj + shadow_thickness;
+      int y_offset = y_adj + shadow_thickness;
 
       p = XIMAGE_INSTANCE (instance);
 
@@ -299,12 +354,15 @@ x_get_button_size (struct frame *f, Lisp_Object window,
 		tb->y = y;						\
 		tb->width = width;					\
 		tb->height = height;					\
+	        tb->border_width = border_width;			\
+	        tb->vertical = vert;					\
 									\
                 if (tb->blank || NILP (tb->up_glyph))			\
 		  {							\
 		    int threed = (EQ (Qt, tb->up_glyph) ? 1 : 0);	\
 		    x_draw_blank_toolbar_button (f, x, y, width,	\
-						 height, threed);	\
+						 height, threed,	\
+						 border_width, vert);	\
 		  }							\
 	        else							\
 		  x_output_toolbar_button (f, button);			\
@@ -350,17 +408,37 @@ x_output_toolbar (struct frame *f, enum toolbar_pos pos)
   struct device *d = XDEVICE (f->device);
   int x, y, bar_width, bar_height, vert;
   int max_pixpos, right_size, right_start, blank_size;
+  int border_width = FRAME_REAL_TOOLBAR_BORDER_WIDTH (f, pos);
   Lisp_Object button, window;
+  Display *dpy = DEVICE_X_DISPLAY (d);
+  Window x_win = XtWindow (FRAME_X_TEXT_WIDGET (f));
+  GC background_gc = FRAME_X_TOOLBAR_BLANK_BACKGROUND_GC (f);
 
   get_toolbar_coords (f, pos, &x, &y, &bar_width, &bar_height, &vert, 1);
   window = FRAME_LAST_NONMINIBUF_WINDOW (f);
 
-  if (vert)
-    max_pixpos = y + bar_height;
-  else
-    max_pixpos = x + bar_width;
+  /* Do the border */
+  XFillRectangle (dpy, x_win, background_gc, x, y,
+		  (vert ? bar_width : border_width),
+		  (vert ? border_width : bar_height));
+  XFillRectangle (dpy, x_win, background_gc,
+		  (vert ? x : x + bar_width - border_width),
+		  (vert ? y + bar_height - border_width : y),
+		  (vert ? bar_width : border_width),
+		  (vert ? border_width : bar_height));
 
-  button = FRAME_TOOLBAR_DATA (f, pos)->toolbar_buttons;
+  if (vert)
+    {
+      max_pixpos = y + bar_height - border_width;
+      y += border_width;
+    }
+  else
+    {
+      max_pixpos = x + bar_width - border_width;
+      x += border_width;
+    }
+
+  button = FRAME_TOOLBAR_BUTTONS (f, pos);
   right_size = 0;
 
   /* First loop over all of the buttons to determine how much room we
@@ -378,7 +456,7 @@ x_output_toolbar (struct frame *f, enum toolbar_pos pos)
       button = tb->next;
     }
 
-  button = FRAME_TOOLBAR_DATA (f, pos)->toolbar_buttons;
+  button = FRAME_TOOLBAR_BUTTONS (f, pos);
 
   /* Loop over the left buttons, updating and outputting them. */
   X_OUTPUT_BUTTONS_LOOP (1);
@@ -406,7 +484,13 @@ x_output_toolbar (struct frame *f, enum toolbar_pos pos)
 	  height = bar_height;
 	}
 
-      x_draw_blank_toolbar_button (f, x, y, width, height, 1);
+      /*
+       * Use a 3D pushright separator only if there isn't a toolbar
+       * border.  A flat separator meshes with the border and looks
+       * better.
+       */
+      x_draw_blank_toolbar_button (f, x, y, width, height, !border_width,
+				   border_width, vert);
 
       if (vert)
 	y += height;
@@ -435,7 +519,7 @@ x_output_toolbar (struct frame *f, enum toolbar_pos pos)
 static void
 x_clear_toolbar (struct frame *f, enum toolbar_pos pos, int thickness_change)
 {
-  Lisp_Object frame = Qnil;
+  Lisp_Object frame;
   struct device *d = XDEVICE (f->device);
   int x, y, width, height, vert;
 
@@ -495,7 +579,7 @@ x_redraw_exposed_toolbar (struct frame *f, enum toolbar_pos pos, int x, int y,
 			  int width, int height)
 {
   int bar_x, bar_y, bar_width, bar_height, vert;
-  Lisp_Object button = FRAME_TOOLBAR_DATA (f, pos)->toolbar_buttons;
+  Lisp_Object button = FRAME_TOOLBAR_BUTTONS (f, pos);
 
   get_toolbar_coords (f, pos, &bar_x, &bar_y, &bar_width, &bar_height,
 		      &vert, 1);
@@ -571,148 +655,70 @@ x_redraw_frame_toolbars (struct frame *f)
 }
 
 
-static void
-x_toolbar_size_changed_in_frame_1 (struct frame *f, enum toolbar_pos pos,
-				   Lisp_Object old_visibility)
-{
-  XtWidgetGeometry req, repl;
-  int newval;
-  int oldval = FRAME_X_OLD_TOOLBAR_SIZE (f, pos);
-
-  if (NILP (f->toolbar_visible_p[pos]))
-    newval = 0;
-  else if (!f->init_finished && !INTP (f->toolbar_size[pos]))
-    /* the size might not be set at all if we're in the process of
-       creating the frame.  Otherwise it better be, and we'll crash
-       out if not. */
-    newval = 0;
-  else
-    {
-      Lisp_Object frame;
-
-      XSETFRAME (frame, f);
-      newval = XINT (Fspecifier_instance (Vtoolbar_size[pos], frame, Qzero,
-					  Qnil));
-    }
-
-  if (oldval == newval)
-    return;
-
-  /* We want the text area to stay the same size.  So, we query the
-     current size and then adjust it for the change in the toolbar
-     size. */
-
-  in_specifier_change_function++;
-  if (!in_resource_setting)
-    /* mirror the value in the frame resources, unless already done. */
-    {
-      Arg al [1];
-      XtSetArg (al [0],
-		pos ==    TOP_TOOLBAR ? XtNtopToolBarHeight    :
-		pos == BOTTOM_TOOLBAR ? XtNbottomToolBarHeight :
-		pos ==   LEFT_TOOLBAR ? XtNleftToolBarWidth    :
-		XtNrightToolBarWidth,
-		newval);
-      XtSetValues (FRAME_X_TEXT_WIDGET (f), al, 1);
-    }
-  if (XtIsRealized (FRAME_X_CONTAINER_WIDGET (f)))
-    {
-      int change = newval - oldval;
-      Lisp_Object new_visibility = f->toolbar_visible_p[pos];
-
-      req.request_mode = 0;
-      /* the query-geometry method looks at the current value of
-	 f->toolbar_size[pos], so temporarily set it back to the old
-	 one.  If we were called because of a visibility change we
-	 also have to temporarily restore its old status as well. */
-      f->toolbar_size[pos] = make_int (oldval);
-      if (!EQ (old_visibility, Qzero))
-	f->toolbar_visible_p[pos] = old_visibility;
-      XtQueryGeometry (FRAME_X_CONTAINER_WIDGET (f), &req, &repl);
-      f->toolbar_size[pos] = make_int (newval);
-      if (!EQ (old_visibility, Qzero))
-	f->toolbar_visible_p[pos] = new_visibility;
-
-      if (change < 0)
-	{
-	  x_clear_toolbar (f, pos, change);
-	  mark_frame_toolbar_buttons_dirty (f, pos);
-	}
-      if (pos == LEFT_TOOLBAR || pos == RIGHT_TOOLBAR)
-	repl.width += change;
-      else
-	repl.height += change;
-
-      MARK_FRAME_WINDOWS_STRUCTURE_CHANGED (f);
-      EmacsManagerChangeSize (FRAME_X_CONTAINER_WIDGET (f), repl.width,
-			      repl.height);
-    }
-  /* #### should this go within XtIsRealized()?  probably not ... */
-  FRAME_X_OLD_TOOLBAR_SIZE (f, pos) = newval;
-  in_specifier_change_function--;
-}
-
-static void
-x_toolbar_size_changed_in_frame (struct frame *f, enum toolbar_pos pos,
-				 Lisp_Object oldval)
-{
-  x_toolbar_size_changed_in_frame_1 (f, pos, Qzero);
-}
-
-static void
-x_toolbar_visible_p_changed_in_frame (struct frame *f, enum toolbar_pos pos,
-				      Lisp_Object oldval)
-{
-  x_toolbar_size_changed_in_frame_1 (f, pos, oldval);
-}
-
 static void
 x_initialize_frame_toolbar_gcs (struct frame *f)
 {
   EmacsFrame ef = (EmacsFrame) FRAME_X_TEXT_WIDGET (f);
+  EmacsFramePart *efp = &(ef->emacs_frame);
   XGCValues gcv;
   unsigned long flags = (GCForeground | GCBackground | GCGraphicsExposures);
 
-  gcv.foreground = ef->emacs_frame.background_toolbar_pixel;
+  /*
+   * If backgroundToolBarColor is specified, use it.
+   * Otherwise use the background resource.
+   */
+  if (efp->background_toolbar_pixel == (Pixel) (-1))
+    efp->background_toolbar_pixel = efp->background_pixel;
+
+  /*
+   * ####
+   * If foregroundToolBarColor is specified, use it.
+   * Otherwise use the foreground resource.
+   *
+   * The foreground pixel is currently unused, but will likely be
+   * used when toolbar captions are generated by the toolbar code
+   * instead being incorporated into the icon image.
+   */
+  if (efp->foreground_toolbar_pixel == (Pixel) (-1))
+    efp->foreground_toolbar_pixel = efp->foreground_pixel;
+
+  gcv.foreground = efp->background_toolbar_pixel;
   gcv.background = ef->core.background_pixel;
   gcv.graphics_exposures = False;
   FRAME_X_TOOLBAR_BLANK_BACKGROUND_GC (f) =
     XtGetGC ((Widget) ef, flags, &gcv);
 
-  if (ef->emacs_frame.top_toolbar_shadow_pixel ==
-      ef->emacs_frame.bottom_toolbar_shadow_pixel)
+  if (efp->top_toolbar_shadow_pixel == efp->bottom_toolbar_shadow_pixel)
     {
-      ef->emacs_frame.top_toolbar_shadow_pixel =
-	ef->emacs_frame.background_toolbar_pixel;
-      ef->emacs_frame.bottom_toolbar_shadow_pixel =
-	ef->emacs_frame.background_toolbar_pixel;
+      efp->top_toolbar_shadow_pixel    = efp->background_toolbar_pixel;
+      efp->bottom_toolbar_shadow_pixel = efp->background_toolbar_pixel;
     }
 
-  x_generate_shadow_pixels (f, &ef->emacs_frame.top_toolbar_shadow_pixel,
-			    &ef->emacs_frame.bottom_toolbar_shadow_pixel,
-			    ef->emacs_frame.background_toolbar_pixel,
+  x_generate_shadow_pixels (f, &efp->top_toolbar_shadow_pixel,
+			    &efp->bottom_toolbar_shadow_pixel,
+			    efp->background_toolbar_pixel,
 			    ef->core.background_pixel);
 
-  gcv.foreground = ef->emacs_frame.top_toolbar_shadow_pixel;
+  gcv.foreground = efp->top_toolbar_shadow_pixel;
   gcv.background = ef->core.background_pixel;
   gcv.graphics_exposures = False;
   flags = GCForeground | GCBackground | GCGraphicsExposures;
-  if (ef->emacs_frame.top_toolbar_shadow_pixmap)
+  if (efp->top_toolbar_shadow_pixmap)
     {
       gcv.fill_style = FillOpaqueStippled;
-      gcv.stipple = ef->emacs_frame.top_toolbar_shadow_pixmap;
+      gcv.stipple = efp->top_toolbar_shadow_pixmap;
       flags |= GCStipple | GCFillStyle;
     }
   FRAME_X_TOOLBAR_TOP_SHADOW_GC (f) = XtGetGC ((Widget) ef, flags, &gcv);
 
-  gcv.foreground = ef->emacs_frame.bottom_toolbar_shadow_pixel;
+  gcv.foreground = efp->bottom_toolbar_shadow_pixel;
   gcv.background = ef->core.background_pixel;
   gcv.graphics_exposures = False;
   flags = GCForeground | GCBackground | GCGraphicsExposures;
-  if (ef->emacs_frame.bottom_toolbar_shadow_pixmap)
+  if (efp->bottom_toolbar_shadow_pixmap)
     {
       gcv.fill_style = FillOpaqueStippled;
-      gcv.stipple = ef->emacs_frame.bottom_toolbar_shadow_pixmap;
+      gcv.stipple = efp->bottom_toolbar_shadow_pixmap;
       flags |= GCStipple | GCFillStyle;
     }
   FRAME_X_TOOLBAR_BOTTOM_SHADOW_GC (f) = XtGetGC ((Widget) ef, flags, &gcv);
@@ -750,10 +756,10 @@ x_release_frame_toolbar_gcs (struct frame *f)
   XtReleaseGC (ew, FRAME_X_TOOLBAR_BOTTOM_SHADOW_GC (f));
 
   /* Seg fault if we try and use these again. */
-  FRAME_X_TOOLBAR_BLANK_BACKGROUND_GC (f)  = (GC) - 1;
+  FRAME_X_TOOLBAR_BLANK_BACKGROUND_GC  (f) = (GC) - 1;
   FRAME_X_TOOLBAR_PIXMAP_BACKGROUND_GC (f) = (GC) - 1;
-  FRAME_X_TOOLBAR_TOP_SHADOW_GC (f)        = (GC) - 1;
-  FRAME_X_TOOLBAR_BOTTOM_SHADOW_GC (f)     = (GC) - 1;
+  FRAME_X_TOOLBAR_TOP_SHADOW_GC        (f) = (GC) - 1;
+  FRAME_X_TOOLBAR_BOTTOM_SHADOW_GC     (f) = (GC) - 1;
 }
 
 static void
@@ -762,11 +768,8 @@ x_initialize_frame_toolbars (struct frame *f)
   EmacsFrame ef = (EmacsFrame) FRAME_X_TEXT_WIDGET (f);
 
   if (ef->emacs_frame.toolbar_shadow_thickness < MINIMUM_SHADOW_THICKNESS)
-    {
-      Arg al [1];
-      XtSetArg (al [0], XtNtoolBarShadowThickness, MINIMUM_SHADOW_THICKNESS);
-      XtSetValues (FRAME_X_TEXT_WIDGET (f), al, 1);
-    }
+    Xt_SET_VALUE (FRAME_X_TEXT_WIDGET (f),
+		  XtNtoolBarShadowThickness, MINIMUM_SHADOW_THICKNESS);
 
   x_initialize_frame_toolbar_gcs (f);
 }
@@ -793,6 +796,4 @@ console_type_create_toolbar_x (void)
   CONSOLE_HAS_METHOD (x, output_toolbar_button);
   CONSOLE_HAS_METHOD (x, redraw_exposed_toolbars);
   CONSOLE_HAS_METHOD (x, redraw_frame_toolbars);
-  CONSOLE_HAS_METHOD (x, toolbar_size_changed_in_frame);
-  CONSOLE_HAS_METHOD (x, toolbar_visible_p_changed_in_frame);
 }
