@@ -28,7 +28,6 @@ Boston, MA 02111-1307, USA.  */
 #include "lisp.h"
 
 #include <X11/Xlib.h>
-#include TT_C_H_PATH
 
 #include "buffer.h"
 #include "elhash.h"
@@ -151,13 +150,6 @@ struct Lisp_Tooltalk_Message
   Tt_message m;
 };
 
-static Lisp_Object mark_tooltalk_message (Lisp_Object, void (*) (Lisp_Object));
-static void print_tooltalk_message (Lisp_Object, Lisp_Object, int);
-DEFINE_LRECORD_IMPLEMENTATION ("tooltalk-message", tooltalk_message,
-                               mark_tooltalk_message, print_tooltalk_message,
-                               0, 0, 0,
-			       struct Lisp_Tooltalk_Message);
-
 static Lisp_Object
 mark_tooltalk_message (Lisp_Object obj, void (*markobj) (Lisp_Object))
 {
@@ -181,13 +173,18 @@ print_tooltalk_message (Lisp_Object obj, Lisp_Object printcharfun,
   write_c_string (buf, printcharfun);
 }
 
+DEFINE_LRECORD_IMPLEMENTATION ("tooltalk-message", tooltalk_message,
+                               mark_tooltalk_message, print_tooltalk_message,
+                               0, 0, 0,
+			       struct Lisp_Tooltalk_Message);
+
 static Lisp_Object
 make_tooltalk_message (Tt_message m)
 {
+  Lisp_Object val;
   struct Lisp_Tooltalk_Message *msg =
     alloc_lcrecord_type (struct Lisp_Tooltalk_Message,
 			 lrecord_tooltalk_message);
-  Lisp_Object val;
 
   msg->m = m;
   msg->callback = Qnil;
@@ -227,13 +224,6 @@ struct Lisp_Tooltalk_Pattern
   Tt_pattern p;
 };
 
-static Lisp_Object mark_tooltalk_pattern (Lisp_Object, void (*) (Lisp_Object));
-static void print_tooltalk_pattern (Lisp_Object, Lisp_Object, int);
-DEFINE_LRECORD_IMPLEMENTATION ("tooltalk-pattern", tooltalk_pattern,
-                               mark_tooltalk_pattern, print_tooltalk_pattern,
-                               0, 0, 0,
-			       struct Lisp_Tooltalk_Pattern);
-
 static Lisp_Object
 mark_tooltalk_pattern (Lisp_Object obj, void (*markobj) (Lisp_Object))
 {
@@ -257,6 +247,11 @@ print_tooltalk_pattern (Lisp_Object obj, Lisp_Object printcharfun,
   write_c_string (buf, printcharfun);
 }
 
+DEFINE_LRECORD_IMPLEMENTATION ("tooltalk-pattern", tooltalk_pattern,
+                               mark_tooltalk_pattern, print_tooltalk_pattern,
+                               0, 0, 0,
+			       struct Lisp_Tooltalk_Pattern);
+
 static Lisp_Object
 make_tooltalk_pattern (Tt_pattern p)
 {
@@ -327,7 +322,7 @@ This function is the process handler for the ToolTalk connection process.
   UNGCPRO;
 
   /* see comment in event-stream.c about this return value. */
-  return make_int (0);
+  return Qzero;
 }
 
 static Tt_callback_action
@@ -526,7 +521,7 @@ tt_message_arg_ival_string (Tt_message m, int n)
   int value;
 
   check_status (tt_message_arg_ival (m, n, &value));
-  sprintf (buf, "%d", value);
+  long_to_string (buf, value);
   return build_string (buf);
 }
 
@@ -542,8 +537,8 @@ tt_message_arg_bval_vector (Tt_message m, int n)
   return make_string (value, len);
 }
 
-DEFUN ("get-tooltalk-message-attribute",
-       Fget_tooltalk_message_attribute, 2, 3, 0, /*
+DEFUN ("get-tooltalk-message-attribute", Fget_tooltalk_message_attribute,
+       2, 3, 0, /*
 Return the indicated Tooltalk message attribute.  Attributes are
 identified by symbols with the same name (underscores and all) as the
 suffix of the Tooltalk tt_message_<attribute> function that extracts the value.
@@ -684,8 +679,8 @@ value returned by 'arg_bval like a string is fine.
   return Qnil;
 }
 
-DEFUN ("set-tooltalk-message-attribute",
-       Fset_tooltalk_message_attribute, 3, 4, 0, /*
+DEFUN ("set-tooltalk-message-attribute", Fset_tooltalk_message_attribute,
+       3, 4, 0, /*
 Initialize one Tooltalk message attribute.
 
 Attribute names and values are the same as for
@@ -1426,6 +1421,9 @@ Unprocessed messages are messages that didn't match any patterns.
   Tooltalk_Message_plist_str = build_string ("Tooltalk Message plist");
   Tooltalk_Pattern_plist_str = build_string ("Tooltalk Pattern p plist");
 
+  staticpro(&Tooltalk_Message_plist_str);
+  staticpro(&Tooltalk_Pattern_plist_str);
+
 #define MAKE_CONSTANT(name) do { \
     defsymbol (&Q_ ## name, #name); \
     Fset (Q_ ## name, make_int (name)); \
